@@ -33,6 +33,8 @@
 #define LINE_SIZE 132  /* Thus says abinit */
 #define SLINE_SIZE 300 /* For Siesta, N.B. SLINE_SIZE >= LINE_SIZE */
 
+#define abi_tokenmatch(a,b) (tokenmatch(a,b)&&tokenmatch(a,b "1"))
+
 static int ab_len_read(char *line, int off, double *data, int n,
 		       FILE* infile);
 static int ab_energy_read(char *line, int off, double *data, int n,
@@ -60,13 +62,14 @@ void abinit_in_read(FILE* in, struct unit_cell *c, struct contents *m,
 		    struct kpts *k, struct symmetry *s, struct es *e){
   
   double *acell,*rprim,*scalecart,*znucl,*xcart,*xred,*kpt,*wtk,*shiftk;
-  double *ecut,*toldfe,*spinat,*tnons,*kptnrm,*zion;
+  double *ecut,*toldfe,*spinat,*tnons,*kptnrm,*zion,*kptbounds,*kptbounds2;
   double x,xa,xc,angles[3],mat[3][3],abc[6];
   char line[LINE_SIZE+1];
   char *ptr,*p2;
-  int i,j,jj,success,ignore;
+  int i,j,jj,n,success,ignore;
   int *natom,*typat,*ntypat,*nkpt,*ngkpt,*nshiftk,*kptopt,*ndtset;
   int *nspden,*nsppol,*nsym,*symrel,*natrd,*spgroup,*spgaxor,*spgorig;
+  int *kptopt2,*ndivk,*ndivk2;
   int hall;
   struct infiles *files;
   FILE *infile;
@@ -78,9 +81,10 @@ void abinit_in_read(FILE* in, struct unit_cell *c, struct contents *m,
   struct unit_cell dummy_cell;
   
   acell=rprim=scalecart=znucl=xcart=xred=kpt=wtk=shiftk=NULL;
-  ecut=toldfe=spinat=tnons=kptnrm=zion=NULL;
-  natom=typat=ntypat=nkpt=ngkpt=nshiftk=kptopt=ndtset=NULL;
+  ecut=toldfe=spinat=tnons=kptnrm=zion=kptbounds=kptbounds2=NULL;
+  natom=typat=ntypat=nkpt=ngkpt=nshiftk=kptopt=ndtset=kptopt2=NULL;
   nspden=nsppol=nsym=symrel=natrd=spgroup=spgaxor=spgorig=NULL;
+  ndivk=ndivk2=NULL;
   ptr=line;
   *ptr=0;
 
@@ -214,78 +218,100 @@ void abinit_in_read(FILE* in, struct unit_cell *c, struct contents *m,
       if (!ndtset) ndtset=malloc(sizeof(int));
       ptr=line+ab_int_read(line,ptr-line,ndtset,1,infile);
       if (*ndtset!=1)
-	fprintf(stderr,"Warning: reading only common section of abinit"
-		" .in file with multiple datasets\n");
+	fprintf(stderr,"Warning: reading abinit .in files"
+		" with multiple datasets not fully supported\n");
     }
     else if (!tokenmatch(&ptr,"natom")){
       natom=malloc(sizeof(int));
+      if (!natom) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,natom,1,infile);
     }
     else if (!tokenmatch(&ptr,"natrd")){
       natrd=malloc(sizeof(int));
+      if (!natrd) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,natrd,1,infile);
     }
     else if (!tokenmatch(&ptr,"ntypat")){
       ntypat=malloc(sizeof(int));
+      if (!ntypat) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,ntypat,1,infile);
     }
     else if (!tokenmatch(&ptr,"nsppol")){
       nsppol=malloc(sizeof(int));
+      if (!nsppol) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,nsppol,1,infile);
     }
     else if (!tokenmatch(&ptr,"nspden")){
       nspden=malloc(sizeof(int));
+      if (!nspden) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,nspden,1,infile);
     }
-    else if (!tokenmatch(&ptr,"nkpt")){
+    else if (!abi_tokenmatch(&ptr,"nkpt")){
       nkpt=malloc(sizeof(int));
+      if (!nkpt) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,nkpt,1,infile);
     }
-    else if (!tokenmatch(&ptr,"ngkpt")){
+    else if (!abi_tokenmatch(&ptr,"ngkpt")){
       ngkpt=malloc(3*sizeof(int));
+      if (!ngkpt) error_exit("malloc error for three ints");
       ptr=line+ab_int_read(line,ptr-line,ngkpt,3,infile);
     }
-    else if (!tokenmatch(&ptr,"kptopt")){
+    else if (!abi_tokenmatch(&ptr,"kptopt")){
       kptopt=malloc(sizeof(int));
+      if (!kptopt) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,kptopt,1,infile);
     }
-    else if (!tokenmatch(&ptr,"kptnrm")){
+    else if (!tokenmatch(&ptr,"kptopt2")){
+      kptopt2=malloc(sizeof(int));
+      if (!kptopt2) error_exit("malloc error for int");
+      ptr=line+ab_int_read(line,ptr-line,kptopt2,1,infile);
+    }
+    else if (!abi_tokenmatch(&ptr,"kptnrm")){
       kptnrm=malloc(sizeof(double));
+      if (!kptnrm) error_exit("malloc error for double");
       ptr=line+ab_float_read(line,ptr-line,kptnrm,1,infile);
     }
-    else if (!tokenmatch(&ptr,"nshiftk")){
+    else if (!abi_tokenmatch(&ptr,"nshiftk")){
       nshiftk=malloc(sizeof(int));
+      if (!nshiftk) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,nshiftk,1,infile);
     }
     else if (!tokenmatch(&ptr,"nsym")){
       nsym=malloc(sizeof(int));
+      if (!nsym) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,nsym,1,infile);
     }
     else if (!tokenmatch(&ptr,"spgroup")){
       spgroup=malloc(sizeof(int));
+      if (!spgroup) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,spgroup,1,infile);
     }
     else if (!tokenmatch(&ptr,"spgaxor")){
       spgaxor=malloc(sizeof(int));
+      if (!spgaxor) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,spgaxor,1,infile);
     }
     else if (!tokenmatch(&ptr,"spgorig")){
       spgorig=malloc(sizeof(int));
+      if (!spgorig) error_exit("malloc error for int");
       ptr=line+ab_int_read(line,ptr-line,spgorig,1,infile);
     }
-    else if (!tokenmatch(&ptr,"ecut")){
+    else if (!abi_tokenmatch(&ptr,"ecut")){
       if (!ecut) ecut=malloc(sizeof(double));
       else
 	fprintf(stderr,
 		"Warning: ecut appears more than once. Last one used");
+      if (!ecut) error_exit("malloc error for double");
       ptr=line+ab_energy_read(line,ptr-line,ecut,1,infile);
     }
-    else if (!tokenmatch(&ptr,"toldfe")){
+    else if (!abi_tokenmatch(&ptr,"toldfe")){
       toldfe=malloc(sizeof(double));
+      if (!toldfe) error_exit("malloc error for double");
       ptr=line+ab_energy_read(line,ptr-line,toldfe,1,infile);
     }
     else if (!tokenmatch(&ptr,"charge")){
       e->charge=malloc(sizeof(double));
+      if (!e->charge) error_exit("malloc error for double");
       ptr=line+ab_float_read(line,ptr-line,e->charge,1,infile);
     }
     else{
@@ -397,7 +423,8 @@ void abinit_in_read(FILE* in, struct unit_cell *c, struct contents *m,
       if (!wtk) error_exit("malloc error in abinit_in_read");
       ptr=line+ab_float_read(line,ptr-line,wtk,*nkpt,infile);
     }
-    else if (!tokenmatch(&ptr,"shiftk")){
+    else if (!abi_tokenmatch(&ptr,"shiftk")){
+      if (!nshiftk) error_exit("shiftk found without nshiftk");
       shiftk=malloc(*nshiftk*3*sizeof(double));
       if (!shiftk) error_exit("malloc error in abinit_in_read");
       ptr=line+ab_float_read(line,ptr-line,shiftk,3*(*nshiftk),infile);
@@ -414,6 +441,34 @@ void abinit_in_read(FILE* in, struct unit_cell *c, struct contents *m,
       if (!tnons) error_exit("malloc error in abinit_in_read");
       ptr=line+ab_float_read(line,ptr-line,tnons,3*(*nsym),infile);
     }
+    else if (!abi_tokenmatch(&ptr,"kptbounds")){
+      if (!kptopt) error_exit("kptbounds found without kptopt");
+      if (*kptopt>=0) error_exit("kptbounds found with positive kptopt");
+      kptbounds=malloc(3*(1-*kptopt)*sizeof(double));
+      if (!kptbounds) error_exit("malloc error for kptbounds");
+      ptr=line+ab_float_read(line,ptr-line,kptbounds,-3*(1-*kptopt),infile);
+    }
+    else if (!tokenmatch(&ptr,"kptbounds2")){
+      if (!kptopt2) error_exit("kptbounds2 found without kptopt2");
+      if (*kptopt2>=0) error_exit("kptbounds2 found with positive kptopt2");
+      kptbounds2=malloc(3*(1-*kptopt2)*sizeof(double));
+      if (!kptbounds2) error_exit("malloc error for kptbounds2");
+      ptr=line+ab_float_read(line,ptr-line,kptbounds2,3*(1-*kptopt2),infile);
+    }
+    else if (!abi_tokenmatch(&ptr,"ndivk")){
+      if (!kptopt) error_exit("ndivk found without kptopt");
+      if (*kptopt>=0) error_exit("ndivk found with positive kptopt");
+      ndivk=malloc(-*kptopt*sizeof(int));
+      if (!ndivk) error_exit("malloc error for ndivk");
+      ptr=line+ab_int_read(line,ptr-line,ndivk,-*kptopt,infile);
+    }
+    else if (!tokenmatch(&ptr,"ndivk2")){
+      if (!kptopt2) error_exit("ndivk2 found without kptopt2");
+      if (*kptopt2>=0) error_exit("ndivk2 found with positive kptopt2");
+      ndivk2=malloc(-*kptopt2*sizeof(int));
+      if (!ndivk2) error_exit("malloc error for ndivk2");
+      ptr=line+ab_int_read(line,ptr-line,ndivk2,-*kptopt2,infile);
+    }
     else{ /* skip token */
       i=0;
       while (*(ptr+i)&&(!isspace(*(ptr+i)))) i++;
@@ -517,7 +572,7 @@ void abinit_in_read(FILE* in, struct unit_cell *c, struct contents *m,
 	for(j=0;j<3;j++)
 	  m->atoms[i].vspin[j]=spinat[3*i+j];
     }
-    else fprintf(stderr,"Discarding spins\n");
+    else fprintf(stderr,"Discarding atomic spins\n");
     free(spinat);
     spinat=NULL;
   }
@@ -567,6 +622,7 @@ void abinit_in_read(FILE* in, struct unit_cell *c, struct contents *m,
 	for(jj=0;jj<3;jj++)
 	  mat[j][jj]=symrel[9*i+3*jj+j];
       mat_f2a(mat,s->ops[i].mat,c->basis,c->recip);
+      s->ops[i].tr=NULL;
       if (tnons){
 	if ((tnons[3*i]!=0)||(tnons[3*i+1]!=0)||(tnons[3*i+2]!=0)){
 	  s->ops[i].tr=malloc(3*sizeof(double));
@@ -652,6 +708,34 @@ void abinit_in_read(FILE* in, struct unit_cell *c, struct contents *m,
 
   if (kpt) free(kpt);
   if (nkpt) free(nkpt);
+
+  /* kpoint path */
+  
+  if ((kptbounds)||(kptbounds2)){
+    if (!kptbounds){
+      kptbounds=kptbounds2;
+      kptopt=kptopt2;
+      ndivk=ndivk2;
+    }
+    n=1-*kptopt;
+    k->path=malloc(n*sizeof(struct atom));
+    if (!k->path) error_exit("malloc error for path kpts");
+    init_atoms(k->path,n);
+    for(i=0;i<n;i++)
+      for(j=0;j<3;j++)
+	k->path[i].frac[j]=kptbounds[3*i+j];
+    k->path_n=n;
+    if (ndivk){
+      k->path_seg_pts=malloc(n*sizeof(int));
+      if (!k->path_seg_pts) error_exit("malloc error for path nkpts");
+      k->path_seg_pts[0]=1;
+      for(i=0;i<n-1;i++)
+	k->path_seg_pts[i+1]=ndivk[i];
+      free(ndivk);
+    }
+    free(kptbounds);
+    free(kptopt);
+  }
   
   /* misc */
 
@@ -1028,6 +1112,7 @@ void abinit_eig_read(FILE* infile, struct unit_cell *c, struct contents *m,
 	ptr+=itmp;
       }
     }
+    bands_select(m,k,e);
     return;
   }
   
@@ -1101,5 +1186,5 @@ void abinit_eig_read(FILE* infile, struct unit_cell *c, struct contents *m,
       free(newfile);
     }
   }
-      
+  bands_select(m,k,e);
 }
