@@ -1,4 +1,7 @@
 /* Read a CASTEP .bands file. Also a .cell file if found */
+/* Code for Siesta bands file too */
+
+/* MJR (c) 2024 */
 
 #include<stdio.h>
 #include<stdlib.h> /* malloc */
@@ -7,7 +10,7 @@
 
 #include "c2xsf.h"
 
-#define LINE_SIZE 100
+#define LINE_SIZE 200
 void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
                struct kpts *kp, struct symmetry *s);
 
@@ -47,6 +50,7 @@ void bands_read(FILE* infile, struct unit_cell *c, struct contents *m,
       k->n=itmp;
       k->kpts=malloc(itmp*sizeof(struct atom));
       if (!k->kpts) error_exit("Malloc error for kpoints");
+      init_atoms(k->kpts,itmp);
     }
     else if (!strncasecmp(cptr,"Number of spin components",25)){
       if (sscanf(cptr+25,"%d",&itmp)!=1)
@@ -156,6 +160,8 @@ void bands_read(FILE* infile, struct unit_cell *c, struct contents *m,
     }
   }
 
+  bands_select(m,k,e);
+  
   /* See if we can find a .cell file */
 
   if (dict_get(m->dict,"in_file")){
@@ -164,15 +170,16 @@ void bands_read(FILE* infile, struct unit_cell *c, struct contents *m,
       f=fopen(newfile,"r");
       if (f){
 	fprintf(stderr,"Additionally reading %s\n",newfile);
-	kdummy.n=0;
-	kdummy.kpts=NULL;
-	kdummy.mp=NULL;
-	kdummy.spacing=NULL;
+	init_kpts(&kdummy);
 	cell_read(f,c,m,&kdummy,s);
 	fclose(f);
-	if (kdummy.kpts) free(kdummy.kpts);
-	if (kdummy.mp) free(kdummy.mp);
-	if (kdummy.spacing) free(kdummy.spacing);
+	if (kdummy.break_n){
+	  k->breaks=kdummy.breaks;
+	  k->break_n=kdummy.break_n;
+	  kdummy.breaks=NULL;
+	  kdummy.break_n=0;
+	}
+	free_kpts(&kdummy);
       }
     }
     free(newfile);
@@ -187,21 +194,62 @@ static void siesta_bands_read(FILE* infile, struct unit_cell *c,
   int nbands,nspins,nkpts;
   int ik,nb,is,off,i,j;
   char buffer[LINE_SIZE+1],*cptr,*newfile;
-  double dtmp;
+  double dtmp,path;
   FILE *f;
   struct kpts kdummy;
-  
+
   if (debug) fprintf(stderr,"Reading Siesta bands file\n");
   
   if (!fgets(buffer,LINE_SIZE,infile)) error_exit("Unexpected EOF");
-
+  dtmp=0;
+  sscanf(buffer,"%lf",&dtmp);
+  if (dtmp==0)
+    if (!fgets(buffer,LINE_SIZE,infile)) error_exit("Unexpected EOF");
+  
   if (!fgets(buffer,LINE_SIZE,infile)) error_exit("Unexpected EOF");
   if (sscanf(buffer,"%d %d %d",&nbands,&nspins,&nkpts)!=3)
     error_exit("Error parsing third line");
 
+  if (dtmp==0){
+    fprintf(stderr,"Siesta bands file contains lines\n");
+    e->path_eval=malloc(nbands*nspins*nkpts*sizeof(double));
+    if (!e->path_eval) error_exit("malloc error for evals");
+    e->path_nbands=nbands;
+    k->path_nkpt=nkpts;
+    e->nspins=nspins;
+    e->nbspins=nspins;
+
+    path=-1;
+    for(ik=0;ik<nkpts;ik++){
+      if (!fgets(buffer,LINE_SIZE,infile)) error_exit("Unexpected EOF");
+      if (sscanf(buffer,"%lf %n",&dtmp,&off)!=1)
+	error_exit("parse error for kpt");
+      if (dtmp<path)
+	error_exit("path moves backwards");
+      path=dtmp;
+      cptr=buffer+off;
+      for(nb=0;nb<nbands;nb++){
+	for(is=0;is<nspins;is++){
+	  if (sscanf(cptr,"%lf%n",&dtmp,&off)!=1){
+	    if (!fgets(buffer,LINE_SIZE,infile)) error_exit("Unexpected EOF");
+	    cptr=buffer;
+	    if (sscanf(cptr,"%lf%n",&dtmp,&off)!=1) error_exit("Parse error");
+	  }
+	  e->path_eval[nb+nbands*is+nbands*nspins*ik]=dtmp;
+	  cptr+=off;
+	}
+      }
+    }
+    if (debug) fprintf(stderr,"Read %d bands, %d spins, %d kpts\n",
+		       nbands,nspins,nkpts);
+    bands_select(m,k,e);
+    return;
+  }
+
   e->eval=malloc(nbands*nspins*nkpts*sizeof(double));
   e->nbands=nbands;
   e->nspins=nspins;
+  e->nbspins=nspins;
   if (!e->eval) error_exit("malloc error for evals");
   k->n=nkpts;
   k->kpts=malloc(nkpts*sizeof(struct atom));
@@ -210,18 +258,18 @@ static void siesta_bands_read(FILE* infile, struct unit_cell *c,
   for(ik=0;ik<nkpts;ik++){
     if (!fgets(buffer,LINE_SIZE,infile)) error_exit("Unexpected EOF");
     if (sscanf(buffer,"%lf %lf %lf %n",k->kpts[ik].abs,k->kpts[ik].abs+1,
-               k->kpts[ik].abs+2,&off)!=3)
+	       k->kpts[ik].abs+2,&off)!=3)
       error_exit("parse error for kpt");
     cptr=buffer+off;
     for(nb=0;nb<nbands;nb++){
       for(is=0;is<nspins;is++){
-        if (sscanf(cptr,"%lf%n",&dtmp,&off)!=1){
-          if (!fgets(buffer,LINE_SIZE,infile)) error_exit("Unexpected EOF");
-          cptr=buffer;
-          if (sscanf(cptr,"%lf%n",&dtmp,&off)!=1) error_exit("Parse error");
-        }
-        e->eval[nb+nbands*is+nbands*nspins*ik]=dtmp;
-        cptr+=off;
+	if (sscanf(cptr,"%lf%n",&dtmp,&off)!=1){
+	  if (!fgets(buffer,LINE_SIZE,infile)) error_exit("Unexpected EOF");
+	  cptr=buffer;
+	  if (sscanf(cptr,"%lf%n",&dtmp,&off)!=1) error_exit("Parse error");
+	}
+	e->eval[nb+nbands*is+nbands*nspins*ik]=dtmp;
+	cptr+=off;
       }
     }
   }
@@ -230,6 +278,8 @@ static void siesta_bands_read(FILE* infile, struct unit_cell *c,
                      nbands,nspins,nkpts);
 
   if (debug) print_bandwidths(e,k);
+
+  bands_select(m,k,e);
   
   /* See if we can find a .XV file */
   if(!c->basis){
@@ -249,15 +299,10 @@ static void siesta_bands_read(FILE* infile, struct unit_cell *c,
             f=fopen(newfile,"r");
             if (f){
               fprintf(stderr,"Additionally reading %s\n",newfile);
-              kdummy.n=0;
-              kdummy.kpts=NULL;
-              kdummy.mp=NULL;
-              kdummy.spacing=NULL;
+	      init_kpts(&kdummy);
               fdf_read(f,c,m,&kdummy,e);
               fclose(f);
-              if (kdummy.kpts) free(kdummy.kpts);
-              if (kdummy.mp) free(kdummy.mp);
-              if (kdummy.spacing) free(kdummy.spacing);
+	      free_kpts(&kdummy);
             }
           }
         }
@@ -284,3 +329,130 @@ static void siesta_bands_read(FILE* infile, struct unit_cell *c,
   
 }
 
+void bands_select(struct contents *m, struct kpts *k, struct es *e){
+  int ik,is,ib,nk,ns,nb,path=0;
+  int nkpts,nbands,nspins,kn,enb;
+  double *eval,*occ;
+  struct atom *kpt;
+  
+  if ((!e->band_range)||(!e->kpt_range)||(!e->spin_range))
+    return; /* Nothing can be done */
+  
+  if ((!strcmp(e->band_range,"-"))&&(!strcmp(e->kpt_range,"-"))&&
+      (!strcmp(e->spin_range,"-"))) return; /* Nothing to do */
+
+  if (e->path_nbands){
+    path=1;
+    kn=k->path_nkpt;
+    enb=e->path_nbands;
+    fprintf(stderr,"Selecting path bands\n");
+  }
+  else{
+    kn=k->n;
+    enb=e->nbands;
+  }
+  
+  nkpts=0;
+  for(ik=0;ik<kn;ik++)
+    if (inrange(ik+1,e->kpt_range)) nkpts++;
+
+  nspins=0;
+  for(is=0;is<e->nspins;is++)
+    if (inrange(is,e->spin_range)) nspins++;
+
+  nbands=0;
+  for(ib=0;ib<enb;ib++)
+    if (inrange(ib+1,e->band_range)) nbands++;
+
+  if ((nbands==enb)&&(nspins==e->nspins)&&(nkpts==kn)) return;
+
+  if (debug) fprintf(stderr,"Selecting %d bands from %d, %d spins from %d,"
+		     " %d k-points from %d\n",nbands,enb,
+		     nspins,e->nspins,nkpts,kn);
+
+  if (nspins*nbands*nkpts==0) error_exit("impossible null selection");
+  
+  kpt=malloc(nkpts*sizeof(struct atom));
+  eval=malloc(nkpts*nspins*nbands*sizeof(double));
+  occ=malloc(nkpts*nspins*nbands*sizeof(double));
+
+  if ((!kpt)||(!eval)||(!occ))
+    error_exit("malloc error in bands_select");
+
+  nk=0;
+  for(ik=0;ik<kn;ik++){
+    if (!inrange(ik+1,e->kpt_range)) continue;
+    if (path){
+      if (k->path_kpts)
+	kpt[nk]=k->path_kpts[ik];
+    }
+    else
+      kpt[nk]=k->kpts[ik];
+    ns=0;
+    for(is=0;is<e->nspins;is++){
+      if (!inrange(is,e->spin_range)) continue;
+      nb=0;
+      for(ib=0;ib<enb;ib++){
+	if (!inrange(ib+1,e->band_range)) continue;
+	if (path){
+	  if (e->path_eval)
+	    eval[nb+nbands*ns+nbands*nspins*nk]=
+	      e->path_eval[ib+enb*is+enb*e->nspins*ik];
+	  if (e->occ)
+	    occ[nb+nbands*ns+nbands*nspins*nk]=
+	      e->path_occ[ib+enb*is+enb*e->nspins*ik];
+	}
+	else{
+	  if (e->eval)
+	    eval[nb+nbands*ns+nbands*nspins*nk]=
+	      e->eval[ib+enb*is+enb*e->nspins*ik];
+	  if (e->occ)
+	    occ[nb+nbands*ns+nbands*nspins*nk]=
+	      e->occ[ib+enb*is+enb*e->nspins*ik];
+	}
+	nb++;
+      }
+      ns++;
+    }
+    nk++;
+  }
+
+  if (debug>1) fprintf(stderr,"Selected %d bands from %d, %d spins from %d,"
+		     " %d k-points from %d\n",nbands,e->nbands,
+		     nspins,e->nspins,nkpts,k->n);
+
+  if (path){
+    e->path_nbands=nbands;
+    if (e->path_occ){
+      free(e->path_occ);
+      e->path_occ=occ;
+    }
+    if (e->path_eval){
+      free(e->path_eval);
+      e->path_eval=eval;
+    }
+    if (k->path_kpts){
+      k->path_nkpt=nk;
+      free(k->path_kpts);
+      k->path_kpts=kpt;
+    }
+  }
+  else{
+    e->nbands=nbands;
+    e->nspins=nspins;
+    e->nbspins=nspins;
+    if (e->occ){
+      free(e->occ);
+      e->occ=occ;
+    }
+    if (e->eval){
+      free(e->eval);
+      e->eval=eval;
+    }
+    free(k->kpts);
+    k->kpts=kpt;
+    k->n=nkpts;
+  }
+  
+
+}
