@@ -1,7 +1,7 @@
 /* Various utility functions for dealing with basis sets */
 
 
-/* Copyright (c) 2007-2019 MJ Rutter 
+/* Copyright (c) 2007-2025 MJ Rutter 
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -256,7 +256,7 @@ void cart2abc(struct unit_cell *c, struct contents *m, double *abc,
      if we were passed a motif: */
 
   if (m){
-    if (is_rhs(c->basis)==0) make_rhs(c,m,abc,gptr);
+    if (is_rhs(c->basis)==0) make_rhs(c,m,abc,NULL,gptr);
     /* Correct Cartesian vector atomic quantities */
     for(i=0;i<m->n;i++){
       vabs2frac(m->atoms[i].force,c->recip);
@@ -352,8 +352,9 @@ void basis2abc(double b[3][3], double abc[6]){
 }
 
 /* Make a set of axes into a rhs */
+/* NB sym ops are in absolte co-ords, so do not need adjusting */
 void make_rhs(struct unit_cell *c, struct contents *m, double *abc,
-              struct grid *gptr){
+              struct kpts *kp, struct grid *gptr){
   int i,j,k,ic,itmp,preserve_c,npts;
   double dtmp,(*b)[3],*dptr,*dptr2,*new_grid;
 
@@ -373,11 +374,11 @@ void make_rhs(struct unit_cell *c, struct contents *m, double *abc,
    * Exchange second and third fractional co-ords
    */
   if (flags&LHS_FUDGE) {
-    fprintf(stderr,"Need rh set of vectors for abc output.\n"
+    fprintf(stderr,"Need rh set of vectors for abc or Vasp output.\n"
 	    "Have lh set, and exchange prohibitted...\n");
   }
   else {
-    fprintf(stderr,"Need rh set of vectors for abc output. "
+    fprintf(stderr,"Need rh set of vectors. "
 	    "Exchanging basis vectors %d and %d.\n",
 	    (1+preserve_c)%3+1,(2+preserve_c)%3+1);
 
@@ -403,6 +404,56 @@ void make_rhs(struct unit_cell *c, struct contents *m, double *abc,
       abc[(1+preserve_c)%3+3]=abc[(2+preserve_c)%3+3];
       abc[(2+preserve_c)%3+3]=dtmp;
     }
+
+    if (kp){
+      if (kp->n){
+ 	for(i=0;i<kp->n;i++){
+	  dtmp=kp->kpts[i].frac[(1+preserve_c)%3];
+	  kp->kpts[i].frac[(1+preserve_c)%3]=kp->kpts[i].frac[(2+preserve_c)%3];
+	  kp->kpts[i].frac[(2+preserve_c)%3]=dtmp;
+	}
+      }
+      if (kp->mp){
+	itmp=kp->mp->grid[(1+preserve_c)%3];
+	kp->mp->grid[(1+preserve_c)%3]=kp->mp->grid[(2+preserve_c)%3];
+	kp->mp->grid[(2+preserve_c)%3]=itmp;
+	dtmp=kp->mp->disp[(1+preserve_c)%3];
+	kp->mp->disp[(1+preserve_c)%3]=kp->mp->disp[(2+preserve_c)%3];
+	kp->mp->disp[(2+preserve_c)%3]=dtmp;
+      }
+      if (kp->bs_n){
+ 	for(i=0;i<kp->bs_n;i++){
+	  dtmp=kp->bs_kpts[i].frac[(1+preserve_c)%3];
+	  kp->bs_kpts[i].frac[(1+preserve_c)%3]=
+	    kp->bs_kpts[i].frac[(2+preserve_c)%3];
+	  kp->bs_kpts[i].frac[(2+preserve_c)%3]=dtmp;
+	}
+      }
+      if (kp->bs_mp){
+	itmp=kp->bs_mp->grid[(1+preserve_c)%3];
+	kp->bs_mp->grid[(1+preserve_c)%3]=kp->bs_mp->grid[(2+preserve_c)%3];
+	kp->bs_mp->grid[(2+preserve_c)%3]=itmp;
+	dtmp=kp->bs_mp->disp[(1+preserve_c)%3];
+	kp->bs_mp->disp[(1+preserve_c)%3]=kp->bs_mp->disp[(2+preserve_c)%3];
+	kp->bs_mp->disp[(2+preserve_c)%3]=dtmp;
+      }
+      if (kp->path_nkpt){
+ 	for(i=0;i<kp->path_nkpt;i++){
+	  dtmp=kp->path_kpts[i].frac[(1+preserve_c)%3];
+	  kp->path_kpts[i].frac[(1+preserve_c)%3]=
+	    kp->path_kpts[i].frac[(2+preserve_c)%3];
+	  kp->path_kpts[i].frac[(2+preserve_c)%3]=dtmp;
+	}
+      }
+      if (kp->path_n){
+ 	for(i=0;i<kp->path_n;i++){
+	  dtmp=kp->path[i].frac[(1+preserve_c)%3];
+	  kp->path[i].frac[(1+preserve_c)%3]=
+	    kp->path[i].frac[(2+preserve_c)%3];
+	  kp->path[i].frac[(2+preserve_c)%3]=dtmp;
+	}
+      }
+    } /* end if (kp) */
     
     while((gptr)&&(gptr->data)){
       if (debug>1) fprintf(stderr,"Exchanging axes for 3D grid %dx%dx%d\n",
@@ -732,7 +783,20 @@ void add_primitive(struct unit_cell *c, struct contents *m){
   real2rec(c->primitive);
 #endif
 }  
-  
+
+/* How many points of given spacing lie between two points given
+ * in fractional coordinates?
+ */
+int npts_a2b(double a[3], double b[3], double spacing, double basis[3][3]){
+  int i;
+  struct atom atm;
+
+  if (spacing==0) return 0;
+  for(i=0;i<3;i++)
+    atm.frac[i]=b[i]-a[i];
+  addabs(&atm,1,basis);
+  return (int)(ceil(sqrt(vmod2(atm.abs))/spacing));
+}
   
 struct grid *grid_new(struct grid *gptr){
   if (gptr->next) gptr=gptr->next;
@@ -788,7 +852,6 @@ void init_elect(struct es *e){
   e->nel=e->nup_minus_down=0;
   e->max_nplwv=0;
   e->path_eval=e->path_occ=NULL;
-  e->path_kpt=NULL;
   e->path_nbands=0;
 }
 
@@ -829,9 +892,28 @@ void init_kpts(struct kpts *k){
   k->bs_kpts=NULL;
   k->bs_mp=NULL;
   k->bs_spacing=NULL;
+  k->path_nkpt=0;
+  k->path_kpts=NULL;
   k->path_n=0;
   k->path=NULL;
+  k->path_labels=NULL;
+  k->path_seg_pts=NULL;
   k->path_spacing=NULL;
+  k->break_n=0;
+  k->ibreaks=NULL;
+  k->breaks=NULL;
+}
+
+void free_kpts(struct kpts *k){
+  if (k->kpts) free(k->kpts);
+  if (k->mp) free(k->mp);
+  if (k->spacing) free(k->spacing);
+  if (k->bs_kpts) free(k->bs_kpts);
+  if (k->bs_mp) free(k->bs_mp);
+  if (k->bs_spacing) free(k->bs_spacing);
+  if (k->path) free(k->path);
+  if (k->path_spacing) free(k->path_spacing);
+  if (k->breaks) free(k->breaks);
 }
 
 void init_sym(struct symmetry *s){
