@@ -73,7 +73,7 @@ int cspg_op(struct unit_cell *c, struct contents *m, struct symmetry *s,
   double dot,mod1,mod2,theta,axis[3],vtmp[3],vtmp1[3],vtmp2[3];
   double old_basis[3][3],old_frac[3][3],new_basis[3][3];
   double rhs[3][3],tr[3];
-  struct contents *m2;
+  struct contents *m2,m_dummy;
   struct unit_cell *c2;
   struct symmetry stmp;
   
@@ -88,6 +88,11 @@ int cspg_op(struct unit_cell *c, struct contents *m, struct symmetry *s,
 
   if (debug>1) fprintf(stderr,"Calling spglib with op=0x%x\n",op);
 
+  if (!m){
+    init_motif(&m_dummy);
+    m=&m_dummy;
+  }
+  
   if ((op&CSPG_SNAP)||(op&CSPG_PRIM_NR)){
     for(i=0;i<3;i++)
       for(j=0;j<3;j++)
@@ -519,19 +524,22 @@ int cspg_op(struct unit_cell *c, struct contents *m, struct symmetry *s,
     sym_basis(s,c);
   }
   
-  free(m->atoms);
-  m->atoms=malloc(m->n*sizeof(struct atom));
-  if (!m->atoms) error_exit("malloc error for atoms in c2x2spg");
-  init_atoms(m->atoms,m->n);
-  for(i=0;i<m->n;i++){
-    for(j=0;j<3;j++)
-      m->atoms[i].frac[j]=spg_pos[i][j];
-    m->atoms[i].atno=auid[spg_type[i]].atno;
-    m->atoms[i].spin=auid[spg_type[i]].spin;
-    m->atoms[i].chg=0;
-    m->atoms[i].label=auid[spg_type[i]].label;
+  if (m->atoms) free(m->atoms);
+  m->atoms=NULL;
+  if (m->n){
+    m->atoms=malloc(m->n*sizeof(struct atom));
+    if (!m->atoms) error_exit("malloc error for atoms in c2x2spg");
+    init_atoms(m->atoms,m->n);
+    for(i=0;i<m->n;i++){
+      for(j=0;j<3;j++)
+	m->atoms[i].frac[j]=spg_pos[i][j];
+      m->atoms[i].atno=auid[spg_type[i]].atno;
+      m->atoms[i].spin=auid[spg_type[i]].spin;
+      m->atoms[i].chg=0;
+      m->atoms[i].label=auid[spg_type[i]].label;
+    }
+    addabs(m->atoms,m->n,c->basis);
   }
-  addabs(m->atoms,m->n,c->basis);
 
   /* SPG is capable of returning fractional co-ordinates of 1,
    * whereas we like zeros */
