@@ -25,6 +25,7 @@
 #include<stdio.h>
 #include<stdlib.h> /* malloc */
 #include<string.h>
+#include<ctype.h>
 #include<errno.h>
 
 #include "c2xsf.h"
@@ -43,18 +44,41 @@ static char **title;
 static int is_onetep;
   
 static void spin_read(char *buff, struct atom *a){
-  while (*buff==' ') buff++;
+  int n;
+  while (isspace(*buff)) buff++;
+  if ((*buff==0)||(*buff=='!')) return;
   if (!strncasecmp(buff,"spin",4)){
     buff+=4;
-    while ((*buff==' ')||(*buff=='=')||(*buff==':')) buff++;
+    while (isspace(*buff)||(*buff=='=')||(*buff==':')) buff++;
     if (*buff=='('){
       buff++;
-      if (sscanf(buff,"%lf %lf %lf",a->vspin,a->vspin+1,a->vspin+2)!=3)
+      if (sscanf(buff,"%lf %lf %lf%n",a->vspin,a->vspin+1,a->vspin+2,&n)!=3)
 	fprintf(stderr,"Warning: failed to parse vector spin on atom\n");
+      buff+=n;
+      while (isspace(*buff)) buff++;
+      if (*buff!=')')
+	fprintf(stderr,"Warning: failed to find ) "
+		"parsing vector spin on atom\n");
+      else buff++;
     }
-    else
-      if (sscanf(buff,"%lf",&a->spin)!=1)
+    else{
+      if (sscanf(buff,"%lf%n",&a->spin,&n)!=1)
 	fprintf(stderr,"Warning: failed to parse spin on atom\n");
+      buff+=n;
+    }
+    while (isspace(*buff)) buff++;
+  }
+  if (!strncasecmp(buff,"label",5)){
+    buff+=5;
+    while (isspace(*buff)||(*buff=='=')||(*buff==':')) buff++;
+    n=1;
+    while (*(buff+n)&&(!isspace(*(buff+n)))) n++;
+    a->label=malloc(n+1);
+    if (!a->label) error_exit("malloc error for atom label");
+    strncpy(a->label,buff,n);
+    a->label[n]=0;
+    buff+=n;
+    spin_read(buff,a);
   }
 }
 
@@ -82,7 +106,7 @@ static int set_length_unit(char *ptr, double *unit){
 void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
                struct kpts *kp, struct symmetry *s){
   int i,j,k,n,n2,frac,nsym,success,nlabels;
-  int smisc_size=0,sblock_size=0;
+  int smisc_size=0,sblock_size=0,in_break;
   double scale,default_scale;
   double lat_abc[6],*dptr;
   char sym[14],*ptr2;
@@ -121,7 +145,8 @@ void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
 
   if (debug>2) fprintf(stderr,"Cell read called\n");
 
-  if (!(c->basis=malloc(72))) error_exit("Malloc error in cellread for c->basis");
+  if (!(c->basis=malloc(9*sizeof(double))))
+    error_exit("Malloc error in cellread for c->basis");
 
   while(cellreadline(buffer,LINE_SIZE,dir,&files)){
     ptr=buffer;
@@ -153,7 +178,7 @@ void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
       if ((ptr[6]=='s')||(ptr[6]=='S')) ptr2++;
       if ((*ptr2=='e')||(*ptr2=='E')) ptr2+=2;
       /* Increment ptr2 over spaces, = and : */
-      while ((*ptr2)&&((*ptr2==' ')||(*ptr2==':')||(*ptr2=='='))) ptr2++;
+      while ((*ptr2)&&(isspace(*ptr2)||(*ptr2==':')||(*ptr2=='='))) ptr2++;
       if (!kp->mp){
         kp->mp=malloc(sizeof(struct mp_grid));
         if (!kp->mp) error_exit("Malloc error for struct mp_grid!");
@@ -185,7 +210,7 @@ void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
       if ((ptr[9]=='s')||(ptr[6]=='S')) ptr2++;
       if ((*ptr2=='e')||(*ptr2=='E')) ptr2+=2;
       /* Increment ptr2 over spaces, = and : */
-      while ((*ptr2)&&((*ptr2==' ')||(*ptr2==':')||(*ptr2=='='))) ptr2++;
+      while ((*ptr2)&&(isspace(*ptr2)||(*ptr2==':')||(*ptr2=='='))) ptr2++;
       if (!kp->bs_mp){
         kp->bs_mp=malloc(sizeof(struct mp_grid));
         if (!kp->bs_mp) error_exit("Malloc error for struct mp_grid!");
@@ -215,7 +240,7 @@ void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
       ptr2=ptr+17;
       if ((ptr[6]=='s')||(ptr[6]=='S')) ptr2++;
       /* Increment ptr2 over spaces, = and : */
-      while ((*ptr2)&&((*ptr2==' ')||(*ptr2==':')||(*ptr2=='='))) ptr2++;
+      while ((*ptr2)&&(isspace(*ptr2)||(*ptr2==':')||(*ptr2=='='))) ptr2++;
       dptr=malloc(sizeof(double));
       if (!dptr) error_exit("Malloc error for single double!");
       if (!cell_read_inv_length(ptr2,dptr)){
@@ -230,7 +255,7 @@ void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
       ptr2=ptr+20;
       if ((ptr[9]=='s')||(ptr[9]=='S')) ptr2++;
       /* Increment ptr2 over spaces, = and : */
-      while ((*ptr2)&&((*ptr2==' ')||(*ptr2==':')||(*ptr2=='='))) ptr2++;
+      while ((*ptr2)&&(isspace(*ptr2)||(*ptr2==':')||(*ptr2=='='))) ptr2++;
       dptr=malloc(sizeof(double));
       if (!dptr) error_exit("Malloc error for single double!");
       if (!cell_read_inv_length(ptr2,dptr)){
@@ -245,7 +270,7 @@ void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
       ptr2=ptr+17;
       if ((ptr[9]=='s')||(ptr[9]=='S')) ptr2++;
       /* Increment ptr2 over spaces, = and : */
-      while ((*ptr2)&&((*ptr2==' ')||(*ptr2==':')||(*ptr2=='='))) ptr2++;
+      while ((*ptr2)&&(isspace(*ptr2)||(*ptr2==':')||(*ptr2=='='))) ptr2++;
       dptr=malloc(sizeof(double));
       if (!dptr) error_exit("Malloc error for single double!");
       if (!cell_read_inv_length(ptr2,dptr)){
@@ -268,10 +293,10 @@ void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
     if (strncasecmp(ptr,"%block",6)) continue;
     ptr+=6;
 /* Eat leading spaces */
-    while(*ptr==' ') ptr++;
+    while(isspace(*ptr)) ptr++;
 /* Kill trailing spaces and things after ! */
     ptr2=ptr;
-    while((*ptr2)&&(*ptr2!=' ')&&(*ptr2!='!')) ptr2++;
+    while((*ptr2)&&(!isspace(*ptr2))&&(*ptr2!='!')) ptr2++;
     *ptr2=0; /* Either it was a null, or it should be one... */
 
     if (debug>2) fprintf(stderr,"Found a %s block\n",ptr);
@@ -492,11 +517,26 @@ void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
       }
       kp->bs_n=i;
     }else if((!strcasecmp(ptr,"bs_kpoint_path"))||
-	     (!strcasecmp(ptr,"bs_kpoints_path"))){
+	     (!strcasecmp(ptr,"bs_kpoints_path"))||
+	     (!strcasecmp(ptr,"spectral_kpoint_path"))||
+	     (!strcasecmp(ptr,"spectral_kpoints_path"))){
       i=0;
+      in_break=0;
       while(1){
         cellreadline(buffer,LINE_SIZE,dir,&files);
         if (!strncasecmp(buffer,"%endblock",9)) break;
+	if (!strncasecmp(buffer,"break",5)){
+	  kp->break_n++;
+	  kp->breaks=realloc(kp->breaks,6*kp->break_n*sizeof(double));
+	  if (!kp->breaks) error_exit("realloc error for kpt breaks");
+	  kp->ibreaks=realloc(kp->ibreaks,kp->break_n*sizeof(int));
+	  if (!kp->ibreaks) error_exit("realloc error for kpt ibreaks");
+	  memcpy(kp->breaks+6*(kp->break_n-1),kp->path[i-1].frac,
+		 3*sizeof(double));
+	  kp->ibreaks[kp->break_n-1]=i;
+	  in_break=1;
+	  continue;
+	}
         kp->path=realloc(kp->path,(i+1)*sizeof(struct atom));
         if (!kp->path) error_exit("realloc error for kpts");
 	init_atoms(kp->path+i,1);
@@ -504,6 +544,11 @@ void cell_read(FILE* in, struct unit_cell *c, struct contents *m,
           PARSE_ERROR;
           exit(1);
         }
+	if (in_break){
+	  memcpy(kp->breaks+6*(kp->break_n-1)+3,kp->path[i].frac,
+		 3*sizeof(double));
+	  in_break=0;
+	}
         i++;
       }
       kp->path_n=i;
@@ -665,12 +710,12 @@ int cellreadline(char *buffer, int len, char *dir, struct infiles **filesp){
 
 /* Kill trailing spaces and newlines / carriage returns */
     ptr=buffer+strlen(buffer)-1;
-    while((ptr>=buffer)&&((*ptr==' ')||(*ptr=='\n')||(*ptr=='\r'))) ptr--;
+    while((ptr>=buffer)&&(isspace(*ptr)||(*ptr=='\n')||(*ptr=='\r'))) ptr--;
     *(ptr+1)=0;
 
 /* Eat leading spaces */
     ptr=buffer;
-    while(*ptr==' ') ptr++;
+    while(isspace(*ptr)) ptr++;
 /* Parse a very special comment */
     if (!strncasecmp(ptr,"#titl ",6)){ /* We have a shelx-style title */
       ptr+=6;
@@ -710,7 +755,7 @@ int cellreadline(char *buffer, int len, char *dir, struct infiles **filesp){
 
   if (!strncasecmp(ptr,"includefile",11)){
     ptr=buffer+11;
-    while ((*ptr==' ')||(*ptr==':')) ptr++;
+    while (isspace(*ptr)||(*ptr==':')) ptr++;
     if (*ptr=='"'){
       ptr++;
       p2=ptr;
@@ -729,7 +774,7 @@ static int cell_read_length(char *buff, double *x){
   char *p;
   int n,i;  
 
-  while((*buff)&&((*buff==' ')||(*buff==':')||(*buff=='='))) buff++;
+  while((*buff)&&(isspace(*buff)||(*buff==':')||(*buff=='='))) buff++;
 
   /*
    *  n=sscanf(buff,"%lf %ms",x,&p);
@@ -774,7 +819,7 @@ static int cell_read_inv_length(char *buff, double *x){
   char *p;
   int n,i;  
 
-  while((*buff)&&((*buff==' ')||(*buff==':')||(*buff=='='))) buff++;
+  while((*buff)&&(isspace(*buff)||(*buff==':')||(*buff=='='))) buff++;
 
   /*
    *  n=sscanf(buff,"%lf %ms",x,&p);
