@@ -5,7 +5,7 @@
  */
 
 
-/* Copyright (c) 2007 -- 2023 MJ Rutter 
+/* Copyright (c) 2007 -- 2025 MJ Rutter 
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -95,10 +95,16 @@ void vmd_recip_write(FILE* outfile, struct unit_cell *c, struct kpts *k);
 void jmol_recip_write(FILE* outfile, struct unit_cell *c, struct kpts *k);
 void vert_write(FILE* outfile, struct poly *p);
 void vert_frac_write(FILE* outfile, struct poly *p, double basis[3][3]);
+void bands_plot(FILE* outfile, struct unit_cell *c, struct contents *m,
+		struct kpts *k, struct es *e, char *fmt);
+void dos_plot(FILE* outfile, struct unit_cell *c, struct contents *m,
+	      struct kpts *k, struct symmetry *s, struct es *e,
+	      char *fmt, double *range);
 
 void file_read(char *file1, FILE* infile, struct unit_cell *c,
 	       struct contents *m, struct kpts *k, struct symmetry *s,
 	       struct grid *g, struct es *elect, struct time_series *ts,
+	       struct kpts *q, struct es *ph,
 	       int *i_grid, int fr_flags);
 void cell_read(FILE* infile, struct unit_cell *c, struct contents *m,
                struct kpts *k, struct symmetry *s);
@@ -142,6 +148,9 @@ void grad2_real(struct grid *g, struct unit_cell *c);
 void vdiv(struct grid *g, struct unit_cell *c);
 void component(struct grid *g, int c);
 void gmod(struct grid *g);
+char *bravais(double b[3][3]);
+char *bravais_sc(double b[3][3], int perm[3]);
+struct pt *bspec_sc(double b[3][3]);
 double interpolate_spherical(struct grid *gptr, int comp, struct unit_cell *c,
 			   double *origin_frac, double len, int n,
 			     double *points, char mode);
@@ -156,7 +165,6 @@ void check_read(FILE* infile, struct unit_cell *c, struct contents *m,
                 struct es *elect, int *i_grid);
 
 
-
 /* Global variables for system description */
 
 int debug,flags;
@@ -192,34 +200,37 @@ int main(int argc, char **argv)
   int molecule=0,reduce=0,vexpand=0,lflags=0,pr_occ=0,pr_gap=0,pr_widths=0;
   int sort_style=0,calc_ef=0,calc_grad=0,firstfile_arg=0,op=0,replace_rho=0;
   int no_sym=0,sym_group_check=0,first_bz=0,kpt_reduce=0,extract_comp=0;
-  int calc_mod=0,vec2force=0,fr_flags=0;
+  int calc_mod=0,vec2force=0,fr_flags=0,have_kpt_range=0;
   int circ[3];
   int spg_op;
-  int gen_mp=0, failure=0,calc_esp=0,sym_k=0,sym_list=0;
+  int gen_mp=0, failure=0,calc_esp=0,sym_k=0,sym_list=0,rhs=0;
   int keep_velocities=0;
   int format,preserve_c,mask;
   int *i_grid,*i_frame,super_grid[3];
   int i_expand[3];
   char *optp,*file1=NULL,*outname=NULL,*line_spec=NULL,*pt_spec=NULL;
-  char *cdfile,*cptr,*v2f_spec=NULL,*v2f_atoms=NULL,ctmp[6];
+  char *cdfile,*cptr,*v2f_spec=NULL,*v2f_atoms=NULL,ctmp[6],*dos_fmt=NULL;
+  char *title=NULL,*band_fmt=NULL;
   FILE *infile,*outfile;
   double abc[6],new_cell[3][3],new_cell_rel[3][3],*new_mp,zpt[3],g_cut;
-  double new_abc[3],dtmp,scale,rescale,radius,v2f_rad;
-  double tolmin=1e-4,ionic_charge,musq,tube_spacing,*m_rel,chg,*vec;
+  double new_abc[3],dtmp,scale,rescale,radius,v2f_rad,*skpt,skpt_store[3];
+  double tolmin=1e-4,ionic_charge=0,musq,tube_spacing,*m_rel,chg,*vec;
+  double *xrange,xrange_store[2];
   int *m_abc;
   struct grid *gptr;
   struct unit_cell cell,cell2,nc;
   struct contents motif,motif2;
   struct symmetry sym,sym2;
-  struct kpts kp,kp2;
+  struct kpts kp,kp2,qp;
   struct grid grid1,grid2;
-  struct es elect,elect2;
+  struct es elect,elect2,phonon;
   struct time_series series,series2;
   
 /* Initialise all pointers to NULL, etc. */
 
   init_elect(&elect);
   init_elect(&elect2);
+  init_elect(&phonon);
   
   init_motif(&motif);
   init_motif(&motif2);
@@ -233,10 +244,13 @@ int main(int argc, char **argv)
 
   init_kpts(&kp);
   kp2=kp;
+  init_kpts(&qp);
   
   m_abc=i_grid=i_frame=NULL;
   m_rel=NULL;
   new_mp=NULL;
+  skpt=NULL;
+  xrange=NULL;
 
   init_grid(&grid1);
   grid2=grid1;
@@ -326,6 +340,24 @@ int main(int argc, char **argv)
           format=CASTEP_BANDS;
           flags|=OCCUPANCIES;
         }
+        else if (!strncmp(optp,"--bands_",8)) {
+          format=BANDS_GNU;
+	  band_fmt=optp;
+          flags|=OCCUPANCIES;
+        }
+        else if (!strncmp(optp,"--phbands_",10)) {
+	  band_fmt=optp;
+          format=BANDS_PH;
+        }
+        else if (!strncmp(optp,"--dos_",6)) {
+          format=DOS_GNU;
+          flags|=OCCUPANCIES;
+	  dos_fmt=optp;
+        }
+        else if (!strncmp(optp,"--phdos_",8)) {
+          format=DOS_PH;
+	  dos_fmt=optp;
+        }
         else if (!strcmp(optp,"--geom")) format=CASTEP_GEOM;
         else if (!strcmp(optp,"--xv")) format=XV;
         else if (!strcmp(optp,"--ccp4")) format=CCP4;
@@ -370,6 +402,16 @@ int main(int argc, char **argv)
         else if (!strcmp(optp,"--sym")) spg_op|=CSPG_SYM;
         else if (!strcmp(optp,"--point")) spg_op|=CSPG_PNT;
         else if (!strcmp(optp,"--list")) spg_op|=(CSPG_LST|CSPG_SYM);
+        else if (!strncmp(optp,"--bravais",9)) {
+	  flags|=BRAVAIS;
+	  if (*(optp+9)=='='){
+	    if (point_scan(optp+10,skpt_store,&n)==1)
+	      skpt=skpt_store;
+	    else
+	      fprintf(stderr,
+		      "Warning, ignored malformed option --bravais=(a,b,c)\n");
+	  }
+	}
 
         else if (!strcmp(optp,"--help")) help();
         else if (!strcmp(optp,"--refs")) refs();
@@ -378,11 +420,13 @@ int main(int argc, char **argv)
         else if (!strcmp(optp,"--formats")) formats();
         else if (!strcmp(optp,"--version")) version();
         else if (!strcmp(optp,"--calc")){
-          single_scan(argv[opt+1],&dtmp,NULL);
+          i=single_scan(argv[opt+1],&dtmp,NULL);
           if (debug) printf("%s = ",argv[opt+1]);
-          printf("%.15g\n",dtmp);
+          if (i) printf("%.15g\n",dtmp);
+	  else printf("Invalid expression\n");
           exit(0);
         }
+	else if (!strncmp(optp,"--title=",8)) title=optp+8;
 	else if (!strcmp(optp,"--fbz")) first_bz=1; 
 	else if (!strcmp(optp,"--ibz")) first_bz=2; 
         else if (!strcmp(optp,"--vel")) keep_velocities=1;
@@ -412,7 +456,12 @@ int main(int argc, char **argv)
 	  if (sscanf(optp+8,"%d",i_frame)!=1)
 	    error_exit("Failed to parse --frame= argument");
 	}
+	else if (!strncmp(optp,"--xrange=",9)) {
+	  if (multi_scan(optp+9,xrange_store,2,&i)==2)
+	    xrange=xrange_store;
+	}
 	else if (!strcmp(optp,"--merge")) op|=C2X_MERGE;
+	else if (!strcmp(optp,"--merge_path")) op|=C2X_MERGE_PATH;
 	else if (!strcmp(optp,"--add")) op=(op&~C2X_EXC_MASK)+C2X_ADD;
 	else if (!strcmp(optp,"--sum")) op=(op&~C2X_EXC_MASK)+C2X_ADD;
 	else if (!strcmp(optp,"--diff")) op=(op&~C2X_EXC_MASK)+C2X_DIFF;
@@ -440,6 +489,7 @@ int main(int argc, char **argv)
 	else if (!strcmp(optp,"--sym_add")) flags|=SYM_ADD;
 	else if (!strcmp(optp,"--sym_del")) flags&=(~SYM_ADD);
 	else if (!strcmp(optp,"--sym_group")) sym_group_check=1;
+	else if (!strcmp(optp,"--rhs")) rhs=1;
 	else {
           fprintf(stderr,"Invalid option %s.\n%s -h for usage.\n",
                    optp,argv[0]);
@@ -572,6 +622,7 @@ int main(int argc, char **argv)
         case 'k':
           if(*(optp+1)=='='){
             elect.kpt_range=optp+2;
+	    have_kpt_range=1;
             while(*((++optp)+1));
           }
           else error_exit("malformed option -k");
@@ -866,6 +917,12 @@ int main(int argc, char **argv)
      }
   }
 
+  /* Default for these formats is all kpts, not first */
+  if (((format==CASTEP_BANDS)||(format==BXSF))&&(!have_kpt_range))
+    elect.kpt_range="-";
+  if ((flags&OCCUPANCIES)&&(!have_kpt_range))
+    elect.kpt_range="-";
+  
   dict_add(motif.dict,"i_frame",i_frame);
   
   if (!file1) error_exit("no input file specified.");
@@ -884,6 +941,9 @@ int main(int argc, char **argv)
   if ((!outname)&&(format==0)&&(spg_op&(CSPG_INT|CSPG_SCH|CSPG_PNT|CSPG_LST)))
     format=CNULL;
 
+  if ((!outname)&&(format==0)&&(flags&BRAVAIS))
+    format=CNULL;
+
   n=argc-firstfile_arg;
   if (outname) n--; /* We have an output file */
   if ((n>1)&&(op==0)) /* If confused, exit before truncating output file */
@@ -938,17 +998,20 @@ int main(int argc, char **argv)
     }
     if (j==0)
       file_read(file1,infile,&cell,&motif,&kp,&sym,&grid1,&elect,&series,
-		i_grid,fr_flags);
+		&qp,&phonon,i_grid,fr_flags);
     else{
       init_cell(&cell2);
       init_kpts(&kp2);
       init_sym(&sym2);
       init_grid(&grid2);
       init_elect(&elect2);
+      elect2.band_range=elect.band_range;
+      elect2.spin_range=elect.spin_range;
+      elect2.kpt_range=elect.kpt_range;
       init_tseries(&series2);
       
       file_read(file1,infile,&cell2,&motif2,&kp2,&sym2,&grid2,&elect2,
-		&series2,i_grid,fr_flags);
+		&series2,NULL,NULL,i_grid,fr_flags);
       if ((j==1)&&(op&C2X_MERGE)){
 	data_combine(C2X_MERGE,&cell,&motif,&kp,&sym,&grid1,&elect,&series,
 		     i_grid,&cell2,&motif2,&kp2,&sym2,&grid2,&elect2,&series2);
@@ -983,10 +1046,16 @@ int main(int argc, char **argv)
     }
   }
 
+  if (title){
+    if (motif.title) free(motif.title);
+    motif.title=title;
+  }
+  
   /* Check that we have most of what we need */
 
   if (!cell.basis){
-    if ((format!=NPY)&&(format!=CNULL)&&(!replace_rho))
+    if ((format!=NPY)&&(format!=CNULL)&&(format!=BANDS_GNU)&&
+	(format!=BANDS_PH)&&(format!=DOS_GNU)&&(!replace_rho))
       error_exit("no basis set found. Was format correctly detected?\n");
   }
   
@@ -998,9 +1067,10 @@ int main(int argc, char **argv)
   if (!motif.atoms){
     if ((format!=DENFMT)&&(format!=CUBE)&&(format!=DX)&&
         (format!=FBIN)&&(format!=BXSF)&&(format!=CASTEP_BANDS)&&
+	(format!=BANDS_GNU)&&(format!=DOS_GNU)&&(format!=DOS_PH)&&
 	(format!=NPY)&&(format!=VMD_RECIP)&&(format!=VMD_WS)&&
 	(format!=VERT)&&(format!=VERT_FRAC)&&(format!=JMOL_RECIP)&&
-	(format!=CNULL)&&(format!=XSF))
+	(format!=CNULL)&&(format!=XSF)&&(format!=BANDS_PH))
       error_exit("no atoms found!");
     else
       if (format==XSF) fprintf(stderr,"Warning: no atoms found\n");
@@ -1045,7 +1115,12 @@ int main(int argc, char **argv)
       fprintf(stderr,"        offset %lf %lf %lf\n",kp.mp->disp[0],
               kp.mp->disp[1],kp.mp->disp[2]);
     }
+    if (kp.n) fprintf(stderr,"%d k-points read\n",kp.n);
+    if (elect.eval) fprintf(stderr,"evals present\n");
+    if (kp.path_nkpt) fprintf(stderr,"%d path k-points read\n",kp.path_nkpt);
+    if (elect.path_eval) fprintf(stderr,"path evals present\n");
     if (sym.n) fprintf(stderr,"%d symmetry operations read\n",sym.n);
+    if (series.nsteps) fprintf(stderr,"%d timesteps read\n",series.nsteps);
   }
 
   if (sym_list&&sym.n){
@@ -1493,13 +1568,7 @@ int main(int argc, char **argv)
   if (sort_style&3) sort_atoms(&motif,sort_style&3);
   if ((sort_style>3)&&(sym.n)) sort_symops(&sym,sort_style);
   if (no_sym){
-    for(i=0;i<sym.n;i++)
-      if (sym.ops[i].tr) free(sym.ops[i].tr);
-    if (sym.ops){
-      free(sym.ops);
-      sym.ops=NULL;
-    }
-    sym.n=0;
+    free_sym(&sym,0);
     if (format==XSF) motif.forces=0;
   }
   if (no_sym==2){
@@ -1556,6 +1625,24 @@ int main(int argc, char **argv)
     check_read(outfile,&cell,&motif,&kp,&sym,&grid1,&elect,NULL);
     exit(0);
   }
+
+  if (flags&BRAVAIS){
+    printf("Bravais lattice type: %s\n",bravais_sc(cell.basis,NULL));
+    (void)bspec_sc(cell.basis);
+    if (skpt){
+      if (kp.path_kpts) free(kp.path_kpts);
+      kp.path_kpts=malloc(sizeof(struct atom));
+      if (!kp.path_kpts) error_exit("malloc error");
+      init_atoms(kp.path_kpts,1);
+      kp.path_nkpt=1;
+      for(i=0;i<3;i++) kp.path_kpts->frac[i]=skpt[i];
+      format=BANDS_GNU;
+      band_fmt="";
+    }
+  }
+
+  if ((rhs)&&(is_rhs(cell.basis)==0))
+    make_rhs(&cell,&motif,NULL,&kp,&grid1);
   
   switch(format){
     case XSF:
@@ -1683,6 +1770,18 @@ int main(int argc, char **argv)
       //      if (!ws_cell.nplanes) zone1(cell.basis,&ws_cell);
       //      vmd_zone_write(outfile,&ws_cell,NULL,cell.basis,NULL);
       //      break;
+    case BANDS_GNU:
+      bands_plot(outfile,&cell,&motif,&kp,&elect,band_fmt);
+      break;
+    case BANDS_PH:
+      bands_plot(outfile,&cell,&motif,&qp,&phonon,band_fmt);
+      break;
+    case DOS_GNU:
+      dos_plot(outfile,&cell,&motif,&kp,&sym,&elect,dos_fmt,xrange);
+      break;
+    case DOS_PH:
+      dos_plot(outfile,&cell,&motif,&qp,&sym,&phonon,dos_fmt,xrange);
+      break;
     case CNULL:
       break;
 
@@ -1703,6 +1802,10 @@ void help(void){
          "-a           rotate as though outputing in abc format\n"
          "-A           accumulate (sum) requested bands\n"
          "-b[=range]   include bands (as psi, rescaled to A^-1.5)\n"
+	 "--bands_eps  band diagram, EPS\n"
+	 "--bands_gnu  band diagram, for gnuplot\n"
+	 "--bravais[=(k1,k2,k3)]\n"
+	 "             report Bravais lattice, identify optional special kpt\n"
          "-B[=range]   include bands (as densities, rescaled to eA^-3)\n"
          "-c           include charge density (rescaled to eA^-3)\n"
 	 "-c=n         extract nth component from vector data, n>=1\n"
@@ -1713,6 +1816,12 @@ void help(void){
          "-d           read also a .chdiff file, constructing its name from\n"
          "               the .cell or .check file given\n"
 	 "--div        convert a vector dataset to its divergence\n"
+	 "--dos_fmt[=s][:n][flags]\n"
+         "             produce DOS plot, fmt=gnu (for gnuplot), eps or raw\n"
+	 "               s=smearing width (eV), n=number of bins\n"
+	 "               flags: a=add spins, g=preserve gaps,"
+	 " G=gaussians not tetrahedra\n"
+	 "                      m=mirror plot, r=rotated plot (eps only)\n"
          "-D=[x,y,z]   calculate dipole moment about fractional co-ordinates\n"
          "               x,y,z (default .5,.5,.5) assuming -c also given\n"
          "-Da=[x,y,z]  ditto, and apply post-hoc slab energy correction with\n"
@@ -1723,6 +1832,7 @@ void help(void){
          "               the .cell or .check file given\n"
          "-e=eps       set tolerance for supercell operations. Default=%g\n"
          "-e=min-max   set tolerance range for spglib operations\n"
+	 "--efermi     calculate Fermi energy from eigenvalues\n"
          "-E[=[-][mu]] calculate electrostatic potential. Default mu=%g\n"
          "-f           find first failure star of k-point mesh\n"
 	 "--fbz        find first Brillouin zone\n"
@@ -1761,7 +1871,7 @@ void help(void){
 	 "--nokinv     do not add inversion to k-space symmetry operations\n"
          "-O           print band occupancies and eigenvalues\n"
 	 "-p           partial charges in cols 61-66 of PDB input\n");
-  printf("-P           find primitive cell\n"
+  printf("-P           find primitive cell (without spglib)\n"
          "-P=X:Y:npts  output 1D data with npts points along line\n"
          "               X and Y either (x,y,z) as fractional co-ords\n"
          "               or atomic symbol followed by atom number\n"
@@ -1783,6 +1893,7 @@ void help(void){
 	 "-r           only reduced (symmetry inequivalent) atoms in cif"
 	 " output\n"
 	 "             or with --fbz/--ibz, reduce kpoints to zone\n"
+	 "--rhs        force right-handed axes by exchanging axes. See -3\n"
          "-R           don't rescale grid data or bands, don't adjust"
 	 " nanotube radius\n"
          "-R=x         rescale grid data by given factor, suffix with an x "
@@ -1799,6 +1910,7 @@ void help(void){
   printf("-t=(x1,y1,z1)(x2,y2,z2)[(x3,y3,z3)]\n"
          "             rotate coords so 1st vector becomes 2nd, using third\n"
          "             vector as axis, else axis is perpendicular to others\n"
+	 "--title=str  set title\n"
 	 "--tricubic   use tricubic interpolation in place of trilinear\n"
          "-T=(x1,y1,z1)(x2,y2,z2)[(x3,y3,z3)]\n"
          "             as -t, but first vec expressed in absolute coords\n"
@@ -1852,7 +1964,8 @@ void help(void){
 	 "  sub or diff   difference corresponding datasets in two input"
 	 " files\n"
 	 "  mask or mult  multiply corresponding datasets in two input files\n"
-	 "  merge    merge data into single output\n\n");
+	 "  merge         merge data into single output\n"
+         "  merge_path    take bandstructure path from second input file\n\n");
 #ifdef SPGLIB
   printf("OPERATION is also used to call spglib and is one of:\n"
          "   primitive         call spg_find_primitive()\n"
@@ -1874,7 +1987,7 @@ void help(void){
 #endif
   printf("Valid values of FORMAT are listed by the --formats argument, as are"
 	 "\nrecognised input formats.\n\n");
-  printf("Version " C2XSF_VER ", (c) MJ Rutter 2007 - 2023"
+  printf("Version " C2XSF_VER ", (c) MJ Rutter 2007 - 2025"
          " licenced under the GPL v3.\n\n");
   printf("If useful to a published paper, please consider citing using the\n"
          "references shown with the --refs argument.\n\n");
@@ -1961,6 +2074,8 @@ void formats(void){
          "            ending .den_fmt are assumed to be in Castep format\n"
 	 "            ending .EIG are assumed to be Abinit or Siesta"
 	 " eigenvalues\n"
+	 "            ending _ELF, _ELF_UP or _ELF_DOWN are assumed to be in\n"
+	 "              Abinit's ELF format,\n"
 	 "            ending .elf or .elf_fmt are assumed to be in Castep ELF"
 	 " format,\n"
 	 "            ending .fdf are assumed to be in Siesta format,\n"
@@ -1990,8 +2105,9 @@ void formats(void){
 	 "              Abinit binary format,\n");
   printf("            ending 3D.OUT are assumed to be in Elk 3D plot format\n");
   printf("            ending or beginning CHG, CHGCAR, POSCAR, CONTCAR, LOCPOT,"
-         " EIGENVAL\n"
-         "              or WAVECAR are assumed to be in VASP 5.x format,\n"
+         " EIGENVAL,\n"
+         "              ELFCAR or WAVECAR are assumed to be in VASP 5.x/6.x "
+	 "format,\n"
 	 "            ending 12 are assumed to be in Crystal format,\n"
 	 "            called fort.34 are assumed to be in Crystal binary "
 	 "format.\n"
@@ -1999,7 +2115,7 @@ void formats(void){
 	 " Elk format\n\n"
          "Otherwise "
          "automatic detection of .cell or .check input. Compatible with\n"
-         ".check files from CASTEP 3.0 to 23.1 (and perhaps beyond).\n\n");
+         ".check files from CASTEP 3.0 to 24.1 (and perhaps beyond).\n\n");
   printf("\nFurther documentation at https://www.c2x.org.uk/\n");
   exit(0);
 }
@@ -2018,13 +2134,17 @@ void refs(void){
 #ifdef SPGLIB
   printf("\nIf making use of the spglib functionality,"
          " consider also citing\n\n");
-  printf("@misc{spglib,\n"
-         "  title = {Spglib, a library for finding and handling "
-         "crystal symmetries},\n"
-         "  author = {Togo, Atsushi},\n"
-         "  url = {https://atztogo.github.io/spglib/}\n"
+  printf("@article{spglib,\n"
+         "  title = {Spglib, a software library for "
+         "crystal symmetry search},\n"
+         "  author = {Togo, Atsushi and Shinohara, Kohei and "
+	 "Tanaka, Isao},\n"
+	 "  journal = {Sci. Technol. Adv. Mater., Meth.},\n"
+	 "  year = {2024},\n"
+	 "  volume = {4},\n"
+	 "  pages = {2384822--2384836},\n"
+	 "  doi = {10.1080/27660400.2024.2384822}\n"
          "}\n");
-  printf("or https://arxiv.org/abs/1808.01590\n");
 #endif
   printf("\nThe post hoc dipole correction schemes are described by:\n\n");
   printf("Slabs:     https://doi.org/10.1103/PhysRevB.46.16067 and\n"
@@ -2033,10 +2153,13 @@ void refs(void){
          "           https://doi.org/10.1103/PhysRevB.60.15476\n");
   printf("Tetragons: https://doi.org/10.1088/1361-648X/ab20e1\n\n");
   printf("\nThe post hoc charge correction schemes are described by:\n\n");
-  printf("0D:        https://link.aps.org/doi/10.1103/PhysRevB.51.4014\n");
+  printf("0D:        https://doi.org/10.1103/PhysRevB.51.4014\n");
   printf("2D:        https://doi.org/10.1088/2516-1075/abeda2\n");
   printf("\nThe irreducible Brillouin zone finder is described by\n");
   printf("           https://doi.org/10.4208/cicp.OA-2021-0094\n");
+  printf("\nBloechl's tetrahedron method for densities of states"
+	 " is described by\n"
+	 "           https://doi.org/10.1103/PhysRevB.49.16223\n");
   exit(0);
 }
 
