@@ -1341,11 +1341,7 @@ static void wave_read(FILE *infile, struct kpts *kp,
     } /* for(k=...) */
   } /* for(ns=...) */
   if ((flags&ACCUMULATE)&&g->data){ /* Now move to a new grid */
-    g->next=malloc(sizeof(struct grid));
-    g=g->next;
-    g->data=NULL;
-    g->next=NULL;
-    g->origin_abs=NULL;
+    g=grid_new(g);
   }    
   if (pwgrid) free(pwgrid);
   if (pwg2)   free(pwg2);
