@@ -1,12 +1,8 @@
 /* Read some useful data from a SIESTA .fdf file
  * 
- */
-
-/* 
  * This version does not parse quotation marks properly.
- */
-
-/* Note that Siesta ignores any of the three characters -_. appearing
+ *
+ * Note that Siesta ignores any of the three characters -_. appearing
  * in its keywords.
  */
 
@@ -28,7 +24,7 @@
  *
  */
 
-/* Copyright (c) 2018-2023 MJ Rutter 
+/* Copyright (c) 2018-2025 MJ Rutter 
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -69,15 +65,17 @@ static int fdfscanfile(char *filename, char *dirname, char *buffer,
 static void fdf_unwind(struct infiles **infile);
 double siesta_pot_charge(char *prefix, char *label); /* Used by xv_read.c */
 void dequote(char **str);
+void bands_read(FILE* infile, struct unit_cell *c, struct contents *m,
+		struct kpts *k, struct symmetry *s,struct es *e);
 
 static char *ptr;
 
 void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
               struct kpts *kp, struct es *e){
-  int i,j,k,n,*atomspecies,mp_grid[3][3],okay,hit;
-  int nspec,*species_to_atno;
-  double scale,shift[3],mp_off[3],theta,phi;
-  char *ptr2,*format;
+  int i,j,jj,k,n,*atomspecies,mp_grid[3][3],grid[3],okay,hit;
+  int nspec,*species_to_atno,band_pi_a=1,tmp,npts;
+  double scale,shift[3],mp_off[3],theta,phi,vtmp[3];
+  char *ptr2,*format,*bandlines=NULL;
   static char buffer[LINE_SIZE+1];
   double *vec,*abc,*atomcoords,*dmetol,latticeconstant,*atomspins;
   double *species_to_charge;
@@ -94,7 +92,8 @@ void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
   char *siesta_block_misc[]={"PAOBasisAndPSlmax","XChybrid",""};
   char *spaces="                              ";  /* 30 spaces */
   char *prefix;
-  char *dir;
+  char *dir,*name;
+  FILE *bandfile;
   struct infiles *files;
  
   dir=dict_get(m->dict,"in_dir");
@@ -173,6 +172,16 @@ void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
       continue;
     }
 
+    if (!fdftokenmatch(&ptr,"BandLinesScale")){
+      while (isspace(*ptr)) ptr++;
+      if (!strncasecmp(ptr,"ReciprocalLatticeVectors",24))
+	band_pi_a=0;
+      else if (!strncasecmp(ptr,"pi/a",4))
+	band_pi_a=1;
+      else
+	fprintf(stderr,"Warning: failed to parse BandLinesScale\n");
+    }
+
     if (!fdftokenmatch(&ptr,"DMEnergyTolerance")){
       dmetol=malloc(sizeof(double));
       if (!fdfreadenergy(ptr,dmetol)){
@@ -215,6 +224,31 @@ void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
 	e->nspins=2;
       continue;
     }
+
+    if (!fdftokenmatch(&ptr,"kgridMonkhorstPack")){
+      if (kp){
+	ptr2=strchr(ptr,'[');
+	if (ptr2){
+	  j=sscanf(ptr2+1,"%d %d %d",grid,grid+1,grid+2);
+	  if (j==3){
+	    kp->mp=malloc(sizeof(struct mp_grid));
+	    if (!kp->mp) error_exit("Malloc error for MP grid");
+	    for(i=0;i<3;i++)
+	      kp->mp->grid[i]=grid[i];
+	    for(i=0;i<3;i++){
+	      kp->mp->disp[i]=0;
+	      if ((kp->mp->grid[i]&1)==0)
+		kp->mp->disp[i]=0.5/kp->mp->grid[i];
+	    }
+	  }
+	  else
+	    fprintf(stderr,"Warning: error parsing kgridMonkhorstPack\n");
+	}
+	else
+	  fprintf(stderr,"Warning: error parsing kgridMonkhorstPack\n");
+      }
+      continue;
+    }
     
     for(i=0;siesta_misc[i][0];i++){
       if (!fdftokenmatch(&ptr,siesta_misc[i])){
@@ -372,6 +406,27 @@ void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
         }
       }
     }
+    else if (!fdfstrcasecmp(&ptr,"BandLines")){
+      while(fdfreadline(buffer,LINE_SIZE,dir,&files)){
+	ptr=buffer;
+	while (isspace(*ptr)) ptr++;
+	if (!strncasecmp(ptr,"%endblock",9)) break;
+	i=strnlen(buffer,LINE_SIZE)+2;
+	if (!bandlines) {
+	  bandlines=malloc(i);
+	  if (!bandlines) error_exit("malloc error for bandlines");
+	  strncpy(bandlines,buffer,i-2);
+	  bandlines[i-2]='\n';
+	  bandlines[i-1]=0;
+	}
+	else{
+	  bandlines=realloc(bandlines,strlen(bandlines)+i);
+	  if (!bandlines) error_exit("realloc error for bandlines");
+	  strncat(bandlines,buffer,i-2);
+	  strcat(bandlines,"\n");
+	}
+      }
+    }
     else{
       hit=0;
       for(i=0;siesta_block_misc[i][0];i++){
@@ -393,6 +448,7 @@ void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
 	fprintf(stderr,"Ignoring %s\n",ptr);
 	while(fdfreadline(buffer,LINE_SIZE,dir,&files)){
 	  ptr=buffer;
+	  while (isspace(*ptr)) ptr++;
 	  if (!strncasecmp(ptr,"%endblock",9)) break;
 	}
       }
@@ -517,8 +573,114 @@ void fdf_read(FILE* in, struct unit_cell *c, struct contents *m,
     free(dmetol);
   }
 
-  free(files);
+  if (bandlines){
+    i=0;
+    ptr=bandlines;
+    while(strchr(ptr,'\n')){
+      ptr=strchr(ptr,'\n')+1;
+      i++;
+    }
+    if (debug>1) fprintf(stderr,"%d bandlines in block\n",i);
+    kp->path_n=i;
+    kp->path=malloc(i*sizeof(struct atom));
+    if (!kp->path) error_exit("malloc error for k->path");
+    init_atoms(kp->path,i);
+    kp->path_labels=malloc(i*sizeof(char*));
+    if (!kp->path_labels) error_exit("malloc error for k->path_labels");
+    kp->path_seg_pts=malloc(i*sizeof(int));
+    if (!kp->path_seg_pts) error_exit("malloc error for k->path_seg_pts");
+    ptr=bandlines;
+    k=0;
+    if (!band_pi_a){
+      for(j=0;j<i;j++){
+	if(sscanf(ptr,"%d %lf %lf %lf%n\n",&npts,
+		  kp->path[j].frac,kp->path[j].frac+1,kp->path[j].frac+2,
+		  &tmp)!=4){
+	  fprintf(stderr,"%s",ptr);
+	  error_exit("parse error for BandLines");
+	}
+	kp->path_seg_pts[j]=npts;
+	ptr+=tmp;
+	while(isspace(*ptr)) ptr++;
+	ptr2=strchr(ptr,'\n');
+	*ptr2=0;
+	kp->path_labels[j]=malloc(strlen(ptr)+1);
+	if (!kp->path_labels) error_exit("malloc error for label");
+	strcpy(kp->path_labels[j],ptr);
+	ptr=ptr2+1;
+      }
+    }
+    else{
+      for(j=0;j<i;j++){
+	if(sscanf(ptr,"%d %lf %lf %lf%n\n",&npts,
+		  kp->path[j].abs,kp->path[j].abs+1,kp->path[j].abs+2,
+		  &tmp)!=4)
+	  error_exit("parse error for pi/a BandLines");
+	for(k=0;k<3;k++)
+	  kp->path[j].abs[k]*=0.5/latticeconstant;
+	kp->path_seg_pts[j]=npts;
+	ptr+=tmp;
+	while(isspace(*ptr)) ptr++;
+	ptr2=strchr(ptr,'\n');
+	*ptr2=0;
+	kp->path_labels[j]=malloc(strlen(ptr)+1);
+	if (!kp->path_labels) error_exit("malloc error for label");
+	strcpy(kp->path_labels[j],ptr);
+	ptr=ptr2+1;
+      }
+      addfrac(kp->path,i,c->basis);
+    }
+  }
   
+  free(files);
+
+  /* Should we attempt to read a .bands file? */
+  if (flags&OCCUPANCIES){
+    if (dict_get(m->dict,"Siesta_SystemLabel"))
+      ptr2=dict_get(m->dict,"Siesta_SystemLabel");
+    else
+      ptr2="siesta";
+    name=malloc(strlen(prefix)+strlen(ptr2)+10);
+    if (!name) error_exit("malloc error for file name");
+    strcpy(name,prefix);
+    strcat(name,ptr2);
+    strcat(name,".bands");
+
+    bandfile=fopen(name,"rb");
+    if (bandfile) fprintf(stderr,"Also reading %s\n",name);
+    else return;
+
+    bands_read(bandfile,c,m,kp,NULL,e);
+
+    if (kp->path_n){
+      n=0;
+      for(i=0;i<kp->path_n;i++)
+	n+=kp->path_seg_pts[i];
+
+      if (n!=kp->path_nkpt){
+	fprintf(stderr,"Warning: input and %s inconsistent %d %d\n",name,
+		kp->path_nkpt,n);
+	return;
+      }
+
+      kp->path_kpts=malloc(n*sizeof(struct atom));
+      if (!kp->path_kpts) error_exit("malloc error for path kpts");
+      init_atoms(kp->path_kpts,n);
+      i=0;
+      for(j=0;j<kp->path_n-1;j++){
+	for(k=0;k<3;k++)
+	  vtmp[k]=(kp->path[j+1].frac[k]-kp->path[j].frac[k])/
+	    kp->path_seg_pts[j+1];
+	for(jj=0;jj<kp->path_seg_pts[j+1];jj++){
+	  for(k=0;k<3;k++)
+	    kp->path_kpts[i].frac[k]=kp->path[j].frac[k]+jj*vtmp[k];
+	  i++;
+	}
+      }
+      for(k=0;k<3;k++)
+	kp->path_kpts[i].frac[k]=kp->path[j].frac[k];
+    }
+  }
 }
 
 int fdfreadline(char *buffer, int len, char *dir, struct infiles **filesp){
