@@ -6,7 +6,7 @@
  */
 
 
-/* Copyright (c) 2007-2019 MJ Rutter 
+/* Copyright (c) 2007-2025 MJ Rutter 
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,7 +36,7 @@ void fdf_write(FILE* fdf, char* filename, struct unit_cell *c,
                struct contents *m, struct kpts *kp,
                struct grid *g, struct es *e){
   int i,j,k,nspec;
-  double dtmp;
+  double dtmp,spacing;
   char *fmt;
 
   fprintf(fdf,"# fdf file created by c2x\n");
@@ -155,6 +155,30 @@ void fdf_write(FILE* fdf, char* filename, struct unit_cell *c,
     fprintf(fdf,"%%endblock kgrid_Monkhorst_Pack\n");
   }
 
+  if ((kp)&&(kp->path_n)){
+    fprintf(fdf,"\nBandLinesScale ReciprocalLatticeVectors\n");
+    fprintf(fdf,"%%block BandLines\n");
+    spacing=0.1;
+    if (kp->path_spacing) spacing=*kp->path_spacing;
+    spacing/=2*M_PI;
+    for(i=0;i<kp->path_n;i++){
+      j=1;
+      if (kp->path_seg_pts)
+	j=kp->path_seg_pts[i];
+      else if (i>0)
+	j=npts_a2b(kp->path[i].frac,kp->path[i-1].frac,
+			spacing,c->recip);
+	
+      fprintf(fdf,"%4d % -13.9g % -13.9g % -13.9g",j,
+	      kp->path[i].frac[0],kp->path[i].frac[1],kp->path[i].frac[2]);
+      if (kp->path_labels)
+	fprintf(fdf,"  %s\n",kp->path_labels[i]);
+      else
+	fprintf(fdf,"\n");
+    }
+    fprintf(fdf,"%%endblock BandLines\n");
+  }
+  
   if (dict_get(m->dict,"Siesta_misc"))
     fprintf(fdf,"\n%s",(char*)dict_get(m->dict,"Siesta_misc"));
   
