@@ -36,7 +36,7 @@ void geom_write(FILE* outfile, struct unit_cell *c, struct contents *m_in,
     ts=malloc(sizeof(struct time_series));
     if (!ts) error_exit("malloc error in geom_write");
     free_ts=1;
-    ts->nsteps=ts->nc=ts->nm=ts->nen=0;
+    init_tseries(ts);
   }
 
   if (ts->nsteps==0){
