@@ -4,7 +4,8 @@
  *
  * This code is copyright MJ Rutter
  *
- * The author is well aware of faster ways of doing FFTs than this version.
+ * The author is well aware of faster ways of doing FFTs than this version,
+ * though this 2025 version is about twice as fast as the previous one.
  *
  * Sign convention is 
  *   dir=-1, exponential is -ikx, generally regarded as forward
@@ -14,7 +15,7 @@
  */
 
 
-/* Copyright (c) 2007, 2015 MJ Rutter 
+/* Copyright (c) 2007, 2015, 2025 MJ Rutter 
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -88,6 +89,10 @@ void fft3d(double* a, int* nn, int dir){
   }
 }
 
+void fft1d(double* a, int n, int dir){
+  m1fft(a,n,1,dir,1,n);
+}
+
 void m1fft(double* a, int n, int str, int dir, int rep, int rstr){
 /* Multiple FFTs, GPFA,
  *
@@ -205,7 +210,8 @@ static void m1spfar(double* a, int p, int n, int dir, int off,int str,
 
 static void m1ftrw(double* a, int n, int nn, int dir, int off, int max, int r,
            int rep, int rstr, double* w){
-/* Multiple DFTs
+/* Multiple DFTs. Done out of place as
+ * we have no idea how big that prime factor is...
  *
  * a -- matrix
  * n -- size
@@ -221,7 +227,7 @@ static void m1ftrw(double* a, int n, int nn, int dir, int off, int max, int r,
 
   int i,j,k,ind,remainder,lots,off2;
   double b[2*BIG];
-  double cr,ci,c0r,c0i,c1r,c1i,tmp;
+  double cr,ci,c0r,c0i,c1r,c1i,tmp,tmp2;
 
   /*
   if (n*rep>BIG){fprintf(stderr,"Increase BIG in m1ftr\n"); exit(1);}
@@ -253,42 +259,84 @@ static void m1ftrw(double* a, int n, int nn, int dir, int off, int max, int r,
   nn*=2;
   rstr*=2;
 
+  /* for performance, a special case for n=2 */
+  if (n==2){
+    ind=off+nn;
+    if (ind>max) ind=ind-max;
+    for(k=0;k<rep;k++){
+      tmp=a[off+k*rstr]-a[ind+k*rstr];
+      tmp2=a[off+k*rstr+1]-a[ind+k*rstr+1];
+      a[off+k*rstr]+=a[ind+k*rstr];
+      a[off+k*rstr+1]+=a[ind+k*rstr+1];
+      a[ind+k*rstr]=tmp*w[0]-tmp2*w[1];
+      a[ind+k*rstr+1]=tmp2*w[0]+tmp*w[1];
+    }
+    return;
+  }
+
+  /* Put DFT into b[] */
+
+  /* i=0 loop */
+  ind=off+nn;
+  if (ind>max) ind=ind-max;
+  /* j=0 and j=1 loops combined */
+  for(k=0;k<rep;k++){
+    b[2*k]=a[off+k*rstr]+a[ind+k*rstr];
+    b[2*k+1]=a[off+k*rstr+1]+a[ind+k*rstr+1];
+  }
+  for(j=2;j<n;j++){
+    ind+=nn;
+    if (ind>max) ind=ind-max;
+    for(k=0;k<rep;k++){
+      b[2*k]+=a[ind+k*rstr];
+      b[2*k+1]+=a[ind+k*rstr+1];
+    }
+  }
 
   c0r=cos(2*(r%n)*M_PI/n);
   c0i=dir*sin(2*(r%n)*M_PI/n);
-
-  c1r=1.0;
-  c1i=0.0;
-
-  for(i=0;i<2*n*rep;i+=2*rep){
-    for(j=0;j<2*rep;j++) b[i+j]=0.0;
-    cr=1.0;
-    ci=0.0;
-    ind=off;
+  c1r=c0r;
+  c1i=c0i;
+  
+  for(i=2*rep;i<2*n*rep;i+=2*rep){
+    cr=c1r;
+    ci=c1i;
+    ind=off+nn;
+    if (ind>max) ind=ind-max;
+    /* j=0 and j=1 loops combined */
     for(k=0;k<rep;k++){
-      b[i+2*k]=0.0;
-      b[i+2*k+1]=0.0;
+      b[i+2*k]=a[off+k*rstr]+a[ind+k*rstr]*cr-a[ind+k*rstr+1]*ci;
+      b[i+2*k+1]=a[off+k*rstr+1]+a[ind+k*rstr]*ci+a[ind+k*rstr+1]*cr;
     }
-    for(j=0;j<n;j++){
-      for(k=0;k<rep;k++){
-        b[i+2*k]+=a[ind+k*rstr]*cr-a[ind+k*rstr+1]*ci;
-        b[i+2*k+1]+=a[ind+k*rstr]*ci+a[ind+k*rstr+1]*cr;
-      }
+    for(j=2;j<n;j++){
       tmp=cr;
       cr=cr*c1r-ci*c1i;
       ci=tmp*c1i+ci*c1r;
       ind+=nn;
       if (ind>max) ind=ind-max;
+      for(k=0;k<rep;k++){
+        b[i+2*k]+=a[ind+k*rstr]*cr-a[ind+k*rstr+1]*ci;
+        b[i+2*k+1]+=a[ind+k*rstr]*ci+a[ind+k*rstr+1]*cr;
+      }
     }
     tmp=c1r;
     c1r=c1r*c0r-c1i*c0i;
     c1i=tmp*c0i+c1i*c0r;
   }
 
-  cr=1.0;
-  ci=0.0;
+  /* Copy from b[] to a[] and multiply by powers of w */
   ind=off;
-  for(i=0;i<2*n*rep;i+=2*rep){
+  /* i=0 loop */
+  for(k=0;k<rep;k++){
+    a[ind+k*rstr]=b[2*k];
+    a[ind+k*rstr+1]=b[2*k+1];
+  }
+  cr=w[0];
+  ci=w[1];
+  ind+=nn;
+  if (ind>max) ind=ind-max;
+  
+  for(i=2*rep;i<2*n*rep;i+=2*rep){
     for(k=0;k<rep;k++){
       a[ind+k*rstr]=b[i+2*k]*cr-b[i+2*k+1]*ci;
       a[ind+k*rstr+1]=b[i+2*k]*ci+b[i+2*k+1]*cr;
@@ -301,8 +349,6 @@ static void m1ftrw(double* a, int n, int nn, int dir, int off, int max, int r,
   }
 }
 
-
-
 static void m1tr(double* a,int n,int nn1,int nn2,int off,int max,int rep, int rstr){
 /* Transpose complex (sub)matrix, stored as real vector
  *
