@@ -121,7 +121,9 @@ int single_scan(char *buff, double *x, int *n){
   /* Try a simple sscanf to see if it consumes whole string */
   i=sscanf(buff,"%lf%n",x,n);
   if ((i>=1)&&((*(buff+(*n))==0)||
-	       (*(buff+(*n))==' ')||(*(buff+(*n))==','))) return 1;
+	       (*(buff+(*n))==' ')||
+	       (*(buff+(*n))==',')||
+	       (*(buff+(*n))==':'))) return 1;
   
   i=sscanf(buff,"%80s%n",s,n);
   if ((i==0)||(i==EOF)) return 0;
@@ -139,7 +141,7 @@ int single_scan(char *buff, double *x, int *n){
 }
 
 /* as single_scan, but have repeat count and read a vector
- * items must be space or comma separated
+ * items must be space, colon or comma separated
  */
 int multi_scan(char *buff, double *x, int rep, int *n){
   int i,n2,dummy;
@@ -153,6 +155,8 @@ int multi_scan(char *buff, double *x, int rep, int *n){
     *n+=n2;
     while (buff[*n]==' ') (*n)++;
     if (buff[*n]==',') (*n)++;
+    if (buff[*n]==':') (*n)++;
+    while (buff[*n]==' ') (*n)++;
   }
 
   return rep;
