@@ -1,4 +1,4 @@
-/* Copyright (c) 2019-2020 MJ Rutter 
+/* Copyright (c) 2019-2023 MJ Rutter 
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -73,7 +73,10 @@ void es_pot(struct unit_cell *c, struct contents *m,
   else{
     if (!grid_size){
       basis2abc(c->basis,abc);
-      for(i=0;i<3;i++) fft[i]=10*abc[i]+1;
+      for(i=0;i<3;i++) {
+	fft[i]=10*abc[i];
+	to235(fft+i);
+      }
       if (debug)
 	fprintf(stderr,"Using grid size of %dx%dx%d\n",fft[0],fft[1],fft[2]);
       for(i=0;i<3;i++) g->size[i]=fft[i];
@@ -94,14 +97,25 @@ void es_pot(struct unit_cell *c, struct contents *m,
       mag+=m->atoms[i].site_chg*m->atoms[i].site_chg;
     }
     if (mag==0){
-      fprintf(stderr,"Cannot calculate ES potential with no electron density\n"
-	      "and no site charges\n");
-      return;
+      fprintf(stderr,"Warning: calculating ES potential with no electron "
+              "density and no site charges\n");
+      for(i=0;i<m->n;i++){
+        net_charge+=m->atoms[i].chg;
+        mag+=m->atoms[i].chg*m->atoms[i].chg;
+      }
+      if (mag==0){
+        fprintf(stderr,"Cannot calculate ES potential with no electron "
+                "density, site charges, or\n"
+                " atomic charges\n");
+        return;
+      }
+    }
+    else{
+      if (fabs(net_charge)>tol)
+        fprintf(stderr,"Warning: net charge of %lf e from site charges!\n",
+                net_charge);
+      dict_strcat(m->dict,"pot_from_site_chg","");
     }
-    if (fabs(net_charge)>tol)
-      fprintf(stderr,"Warning: net charge of %lf e from site charges!\n",
-	      net_charge);
-    dict_strcat(m->dict,"pot_from_site_chg","");
   }
   
   /* Now add ionic potentials */
@@ -205,14 +219,18 @@ void es_pot(struct unit_cell *c, struct contents *m,
     }
   }
 
-  if ((dict_get(m->dict,"charge_correction"))&&(*elect->dip_corr!='m')){
+  if ((dict_get(m->dict,"charge_correction"))&&(*elect->dip_corr)&&
+      (*elect->dip_corr!='m')){
     if (!dipole_ctr){
       dipole_ctr=malloc(3*sizeof(double));
       if (!dipole_ctr) error_exit("Malloc error for three doubles");
       for(i=0;i<3;i++) dipole_ctr[i]=0.5;
     }
     dipole_calc(c,m,g,dipole_ctr,dpole);
-    dipole_slab_dir=elect->dip_corr_dir[0]-'a';
+    if (elect->dip_corr_dir)
+      dipole_slab_dir=elect->dip_corr_dir[0]-'a';
+    else
+      dipole_slab_dir=-1;
     if ((dipole_slab_dir<0)||(dipole_slab_dir>2))
       error_exit("Impossible dipole slab dir");
     ql=0;
@@ -231,7 +249,7 @@ void es_pot(struct unit_cell *c, struct contents *m,
 	for(k=0;k<ngz;k++){
 	  ind[2]=k;
 	  /* in grid cells */
-	  off=(ind[dipole_slab_dir]-0.5*g->size[dipole_slab_dir]);
+	  off=(ind[dipole_slab_dir]-q_off*g->size[dipole_slab_dir]);
 	  /* in A */
 	  off*=ql/g->size[dipole_slab_dir];
 	  /* in eV */
@@ -336,22 +354,31 @@ void ion_recrho(struct unit_cell *c, struct contents *m,
       fprintf(stderr,"\n");
     }
   }
-  
-  for(i=0;i<ngx;i++){
-    ii=i;
-    if (ii>ngx/2) ii=ii-ngx;
-    for(j=0;j<ngy;j++){
-      jj=j;
-      if (jj>ngy/2) jj=jj-ngy;
-      for(k=0;k<ngz;k++){
-        kk=k;
-        if (k>ngz/2) kk=kk-ngz;
-        gvec[0]=ii*c->recip[0][0]+jj*c->recip[1][0]+kk*c->recip[2][0];
-        gvec[1]=ii*c->recip[0][1]+jj*c->recip[1][1]+kk*c->recip[2][1];
-        gvec[2]=ii*c->recip[0][2]+jj*c->recip[1][2]+kk*c->recip[2][2];
-        gsq=4*M_PI*M_PI*(gvec[0]*gvec[0]+gvec[1]*gvec[1]+gvec[2]*gvec[2]);
-        cgrid_bare[k+fft[2]*(j+i*fft[1])]=scale*exp(-gsq/(4*musq));
-        cgrid_ps[k+fft[2]*(j+i*fft[1])]=scale*(1-gsq/(6*musq))*exp(-gsq/(4*musq));
+
+  if (musq==0){
+    for(i=0;i<grid_size;i++){
+      cgrid_bare[i]=scale;
+      cgrid_ps[i]=scale;
+    }
+  }
+  else{
+    for(i=0;i<ngx;i++){
+      ii=i;
+      if (ii>ngx/2) ii=ii-ngx;
+      for(j=0;j<ngy;j++){
+	jj=j;
+	if (jj>ngy/2) jj=jj-ngy;
+	for(k=0;k<ngz;k++){
+	  kk=k;
+	  if (k>ngz/2) kk=kk-ngz;
+	  gvec[0]=ii*c->recip[0][0]+jj*c->recip[1][0]+kk*c->recip[2][0];
+	  gvec[1]=ii*c->recip[0][1]+jj*c->recip[1][1]+kk*c->recip[2][1];
+	  gvec[2]=ii*c->recip[0][2]+jj*c->recip[1][2]+kk*c->recip[2][2];
+	  gsq=4*M_PI*M_PI*(gvec[0]*gvec[0]+gvec[1]*gvec[1]+gvec[2]*gvec[2]);
+	  cgrid_bare[k+fft[2]*(j+i*fft[1])]=scale*exp(-gsq/(4*musq));
+	  cgrid_ps[k+fft[2]*(j+i*fft[1])]=scale*(1-gsq/(6*musq))*
+	    exp(-gsq/(4*musq));
+	}
       }
     }
   }
@@ -475,3 +502,119 @@ void ion_recrho(struct unit_cell *c, struct contents *m,
   free(cgrid_bare);
   
 }
+
+
+/* Be a little lazy, and re-use existing potential and grad functions */
+/* But it is better to add the corrections directly to the field */
+void es_field(struct unit_cell *c, struct contents *m,
+            struct grid *g, struct es *elect, double musq){
+  int i,j,k,ic,ngx,ngy,ngz,offset;
+  int grid_size,ind[3],dipole_slab_dir;
+  double net_charge,off,qa,ql,q_off;
+  double dpole[3],field,*dipole_ctr;
+  double vec[3],mag;
+  void *charge_correction;
+
+  grid_size=g->size[0]*g->size[1]*g->size[2];
+  ngx=g->size[0];
+  ngy=g->size[1];
+  ngz=g->size[2];
+  
+  dipole_ctr=elect->dip_ctr;
+  if (dipole_ctr) dipole_calc(c,m,g,dipole_ctr,dpole);
+  elect->dip_ctr=NULL;
+
+  net_charge=0;
+  charge_correction=dict_get(m->dict,"charge_correction");
+  if (charge_correction){
+    if (g->data){
+      for(i=0;i<grid_size;i++)
+	net_charge-=g->data[i];
+      net_charge*=c->vol/grid_size;
+    }
+    for(i=0;i<m->n;i++)
+      net_charge+=m->atoms[i].chg;
+    if (debug)
+      fprintf(stderr,"Net charge in es_field: %f\n,",net_charge);
+    dict_add(m->dict,"charge_correction",NULL);
+  }
+
+  es_pot(c,m,g,elect,musq);
+  grad(g,c);
+
+  if (dipole_ctr){ /* Need to correct for dipole */
+    dipole_slab_dir=-1;
+    if (elect->dip_corr_dir){
+      dipole_slab_dir=elect->dip_corr_dir[0]-'a';
+    }
+    if ((dipole_slab_dir>=0)&&(dipole_slab_dir<3)){
+      /* Find unit vector in dipole_slab_dir */
+      for(i=0;i<3;i++)
+        vec[i]=c->basis[dipole_slab_dir][i];
+      mag=sqrt(vmod2(vec));
+      for(i=0;i<3;i++)
+        vec[i]*=1/mag;
+      /* Dot dipole with this to find field */
+      field=0;
+      for(i=0;i<3;i++)
+        field+=dpole[i]*vec[i];
+      field=field/(EPS0*c->vol);      
+      fprintf(stderr,"Adding dipole correction of %.3f V/A to potential\n",
+              field);
+      for(ic=0;ic<3;ic++)
+	for(i=0;i<grid_size;i++)
+	  g->data[i+ic*grid_size]+=field*vec[ic];
+    }
+    else{
+      error_exit("Impossible dipole slab dir");
+    }
+  }
+  
+  if ((charge_correction)&&(*elect->dip_corr)&&
+      (*elect->dip_corr!='m')){
+    dipole_slab_dir=-1;
+    if (elect->dip_corr_dir)
+      dipole_slab_dir=elect->dip_corr_dir[0]-'a';
+    if ((dipole_slab_dir<0)||(dipole_slab_dir>2))
+      error_exit("Impossible dipole slab dir");
+    if (!dipole_ctr){
+      dipole_ctr=malloc(3*sizeof(double));
+      if (!dipole_ctr) error_exit("Malloc error for three doubles");
+      for(i=0;i<3;i++) dipole_ctr[i]=0.5;
+    }
+    ql=sqrt(vmod2(c->basis[dipole_slab_dir]));
+    for(i=0;i<3;i++)
+      vec[i]=c->basis[dipole_slab_dir][i]/ql;
+    q_off=dipole_ctr[dipole_slab_dir]+
+      dpole[dipole_slab_dir]/(net_charge*ql);
+    if (debug) fprintf(stderr,"Centre of charge correction: %lf\n",q_off);
+    qa=net_charge/(c->vol*EPS0);
+    for(i=0;i<ngx;i++){
+      ind[0]=i;
+      for(j=0;j<ngy;j++){
+	ind[1]=j;
+	for(k=0;k<ngz;k++){
+	  ind[2]=k;
+	  /* in grid cells */
+	  off=(ind[dipole_slab_dir]-q_off*g->size[dipole_slab_dir]);
+	  /* in A */
+	  off*=ql/g->size[dipole_slab_dir];
+	  /* in V/A */
+	  off=qa*off;
+	  offset=k+ngz*(j+i*ngy);
+	  for(ic=0;ic<3;ic++)
+	    g->data[offset+ic*grid_size]-=off*vec[ic];
+	}
+      }
+    }
+  }
+    
+  elect->dip_ctr=dipole_ctr;
+  dict_add(m->dict,"charge_correction","");
+
+  free(g->name);
+  g->name=malloc(20);
+  if (!g->name) error_exit("malloc error for grid name");
+  sprintf(g->name,"Field, V/A");
+  
+}
