@@ -5,7 +5,7 @@
 
 #include "c2xsf.h"
 
-static void calc_gap(int nspins,struct kpts *k,int nbands,
+static void calc_gap(int nspins, int nkpt, int nbands, struct atom *kpts,
 		     double *eval, int ivb, int use_fermi, double *e_fermi);
 
 void print_basis(double basis[3][3]){
@@ -170,19 +170,19 @@ int band_cmp(const void *ptr1, const void *ptr2){
 struct band { double energy; double weight;};
 
 static struct band *populate_bands(double *eval, int nbands,
-				   int nbspins, struct kpts *kpt){
+				   int nbspins, int nkpt, struct atom *kpts){
   int i,k,b,nb;
   double wt;
   struct band *bands;
 
-  nb=kpt->n*nbspins*nbands;
+  nb=nkpt*nbspins*nbands;
   bands=malloc(nb*sizeof(struct band));
-  if (!bands) error_exit("malloc error in calc_fermi");
+  if (!bands) error_exit("malloc error in populate_bands");
 
   i=0;
-  for(k=0;k<kpt->n;k++){
-    if (kpt->kpts) wt=kpt->kpts[k].wt;
-    else wt=1.0/kpt->n;
+  for(k=0;k<nkpt;k++){
+    if (kpts) wt=kpts[k].wt;
+    else wt=1.0/nkpt;
     if (nbspins==1) wt*=2;
     for(b=0;b<nbspins*nbands;b++){
       bands[i].energy=eval[i];
@@ -210,7 +210,8 @@ double calc_efermi(struct es *elect, struct kpts *kpt, double nel){
   ef=0;
   nb=kpt->n*elect->nbspins*elect->nbands;
 
-  bands=populate_bands(elect->eval,elect->nbands,elect->nbspins,kpt);
+  bands=populate_bands(elect->eval,elect->nbands,elect->nbspins,kpt->n,
+		       kpt->kpts);
   
   i=0;
   total=0;
@@ -248,15 +249,15 @@ double calc_efermi(struct es *elect, struct kpts *kpt, double nel){
 }
 
 /* This version ignores kpt weights */
-double calc_nel(double *eval, int nbands, int nspins, struct kpts *kpt,
+double calc_nel(double *eval, int nbands, int nspins, int nkpt,
 		double e_fermi){
   int i;
   double nel,nb;
   struct band *bands;
   
-  nb=kpt->n*nspins*nbands;
+  nb=nkpt*nspins*nbands;
 
-  bands=populate_bands(eval,nbands,nspins,kpt);
+  bands=populate_bands(eval,nbands,nspins,nkpt,NULL);
 
   i=0;
   while((bands[i].energy<e_fermi)&&(i<nb)) i++;
@@ -269,7 +270,7 @@ double calc_nel(double *eval, int nbands, int nspins, struct kpts *kpt,
 
   free(bands);
   
-  return(nel/kpt->n);
+  return(nel/nkpt);
 }
 
 void print_elect(struct es *elect){
@@ -352,17 +353,17 @@ void print_gap(struct es *elect, struct kpts *kpt, struct contents *m){
 
   if (elect->eval){
     fprintf(stderr,"Band gap from grid\n");
-    calc_gap(elect->nbspins,kpt,elect->nbands,elect->eval,ivb,use_fermi,
-		   elect->e_fermi);
+    calc_gap(elect->nbspins,kpt->n,elect->nbands,kpt->kpts,elect->eval,
+	     ivb,use_fermi,elect->e_fermi);
   }
-  if (elect->path_eval){
+  if ((elect->path_eval)&&(kpt->path_n)){
     fprintf(stderr,"Band gap from line\n");
-    calc_gap(elect->nbspins,elect->path_kpt,elect->path_nbands,
+    calc_gap(elect->nbspins,kpt->path_nkpt,elect->path_nbands,kpt->path_kpts,
 	     elect->path_eval,ivb,use_fermi,elect->e_fermi);
   }
 }
 
-static void calc_gap(int nspins,struct kpts *kpt,int nbands,
+static void calc_gap(int nspins, int nkpt, int nbands, struct atom *kpts,
 		     double *eval, int ivb, int use_fermi, double *e_fermi){
   int k,ns,ind;
   int vbm_ind,cbm_ind,dgap_ind,nel;
@@ -374,10 +375,10 @@ static void calc_gap(int nspins,struct kpts *kpt,int nbands,
   }
 
   if (debug>1) fprintf(stderr,"Calc gap called %d spins %d bands %d kpts\n",
-		       nspins,nbands,kpt->n);
+		       nspins,nbands,nkpt);
 
   if (use_fermi){
-    nel=calc_nel(eval,nbands,nspins,kpt,*e_fermi)+0.5;
+    nel=calc_nel(eval,nbands,nspins,nkpt,*e_fermi)+0.5;
     fprintf(stderr,"Calculated number of electrons %d\n",nel);
     if (nel&1){
       fprintf(stderr,"Odd number of electrons, so metallic\n");
@@ -395,7 +396,7 @@ static void calc_gap(int nspins,struct kpts *kpt,int nbands,
     cbm=1e20;
     dgap=1e20;
     
-    for(k=0;k<kpt->n;k++){
+    for(k=0;k<nkpt;k++){
       ind=k*nspins*nbands+ns*nbands+ivb-1;
       if (use_fermi){
 	if ((eval[ind]>*e_fermi)||
@@ -425,9 +426,9 @@ static void calc_gap(int nspins,struct kpts *kpt,int nbands,
 
     fprintf(stderr,"\nValence band maximum:    ");
     print_energy(vbm);
-    if (kpt->kpts)
-      fprintf(stderr," at k=(%f,%f,%f)\n",kpt->kpts[vbm_ind].frac[0],
-	      kpt->kpts[vbm_ind].frac[1],kpt->kpts[vbm_ind].frac[2]);
+    if (kpts)
+      fprintf(stderr," at k=(%f,%f,%f)\n",kpts[vbm_ind].frac[0],
+	      kpts[vbm_ind].frac[1],kpts[vbm_ind].frac[2]);
     else
       fprintf(stderr,"\n");
 
@@ -437,9 +438,9 @@ static void calc_gap(int nspins,struct kpts *kpt,int nbands,
     
     fprintf(stderr,"Conduction band minimum: ");
     print_energy(cbm);
-    if (kpt->kpts)
-      fprintf(stderr," at k=(%f,%f,%f)\n",kpt->kpts[cbm_ind].frac[0],
-	      kpt->kpts[cbm_ind].frac[1],kpt->kpts[cbm_ind].frac[2]);
+    if (kpts)
+      fprintf(stderr," at k=(%f,%f,%f)\n",kpts[cbm_ind].frac[0],
+	      kpts[cbm_ind].frac[1],kpts[cbm_ind].frac[2]);
     else
       fprintf(stderr,"\n");
 
@@ -450,9 +451,9 @@ static void calc_gap(int nspins,struct kpts *kpt,int nbands,
       fprintf(stderr,"Direct gap:              ");
       print_energy(dgap);
 
-      if (kpt->kpts)
-	fprintf(stderr," at k=(%f,%f,%f)\n",kpt->kpts[dgap_ind].frac[0],
-		kpt->kpts[dgap_ind].frac[1],kpt->kpts[dgap_ind].frac[2]);
+      if (kpts)
+	fprintf(stderr," at k=(%f,%f,%f)\n",kpts[dgap_ind].frac[0],
+		kpts[dgap_ind].frac[1],kpts[dgap_ind].frac[2]);
       else
 	fprintf(stderr,"\n");
 
