@@ -1,6 +1,6 @@
 /* Read a PWscf input file */
 
-/* Copyright (c) 2018, 2021 MJ Rutter 
+/* Copyright (c) 2018--2025 MJ Rutter 
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -638,8 +638,33 @@ void qe_read(FILE* infile, struct unit_cell *c, struct contents *m,
           /* Move to next line */
           *ptr=0;
         }
+	else if (!strncasecmp(ptr,"crystal_b",9)){
+	  qe_readline(&ptr,infile);
+	  if (sscanf(ptr,"%d",&k->path_n)!=1)
+            error_exit("Error parsing kpoints");
+	  k->path=malloc(k->path_n*sizeof(struct atom));
+          if (!k->path) error_exit("Malloc error for kpoints");
+	  init_atoms(k->path,k->path_n);
+	  k->path_seg_pts=malloc(k->path_n*sizeof(int));
+          if (!k->path_seg_pts) error_exit("Malloc error for seg lengths");
+	  k->path_seg_pts[0]=1;
+          for(i=0;i<k->path_n-1;i++){
+            qe_readline(&ptr,infile);
+            if (sscanf(ptr,"%lf %lf %lf %d",k->path[i].frac,
+                       k->path[i].frac+1,
+		       k->path[i].frac+2,
+		       k->path_seg_pts+i+1)!=4)
+              error_exit("Error parsing kpoint");
+          }
+	  qe_readline(&ptr,infile);
+	  if (sscanf(ptr,"%lf %lf %lf",k->path[i].frac,
+		     k->path[i].frac+1,
+		     k->path[i].frac+2)!=3)
+	    error_exit("Error parsing final kpoint");
+          /* Move to next line */
+          *ptr=0;
+	}
         else if ((!strncasecmp(ptr,"tpiba",5))||
-                 (!strncasecmp(ptr,"crystal_b",9))||
                  (!strncasecmp(ptr,"crystal_c",9))){
           fprintf(stderr,"Unable to parse kpoints card of type %s\n",ptr);
           qe_readline(&ptr,infile);
@@ -909,7 +934,7 @@ void qe_read(FILE* infile, struct unit_cell *c, struct contents *m,
     beta=acos(celldm[5]);
     if (abc) beta=M_PI*abc[4]/180;
     c->basis[0][0]=x/2;
-    c->basis[0][2]=y/2;
+    c->basis[0][1]=y/2;
     c->basis[1][0]=-x/2;
     c->basis[1][1]=y/2;
     c->basis[2][0]=z*cos(beta);
