@@ -44,7 +44,7 @@ void qe_xml_read(FILE* infile, char *filename, struct unit_cell *c,
                  int *i_grid){
   char buffer[LINE_SIZE+1];
   char key[6];
-  char *cptr,*cptr2,*cptr3,*den_name,*den_name2,*cwd;
+  char *cptr,*cptr2,*cptr3,*den_name,*den_name2,*cwd,*calculation;
   int i,j,jj,tmp,vspin;
   FILE *ch,*pot;
   struct sp {char *name; double mass; char *p_pot;
@@ -63,6 +63,7 @@ void qe_xml_read(FILE* infile, char *filename, struct unit_cell *c,
   symrel=NULL;
   symtr=NULL;
   species=NULL;
+  calculation=NULL;
   nbands=0;
   nkpt=0;
   /* Be optimistic */
@@ -156,6 +157,7 @@ void qe_xml_read(FILE* infile, char *filename, struct unit_cell *c,
 	strncpy(cptr3,cptr,cptr2-cptr);
 	cptr3[cptr2-cptr]=0;
 	dict_add(m->dict,"QE_calculation",cptr3);
+	calculation=cptr3;
       }
     }
     else if (strstr(buffer,"<forc_conv_thr>")){
@@ -311,6 +313,10 @@ void qe_xml_read(FILE* infile, char *filename, struct unit_cell *c,
       else
         *e->e_fermi*=H_eV;
     }
+    else if (strstr(buffer,"<nelec>")){
+      cptr=strstr(buffer,"<nelec>");
+      sscanf(cptr+7,"%lf",&e->nel);
+    }
     else if(strstr(buffer,"<ks_energies>")){
       if (k->n==0) error_exit("ks_energies before nks");
       while(!strstr(buffer,"</ks_energies>")){
@@ -714,7 +720,6 @@ void qe_xml_read(FILE* infile, char *filename, struct unit_cell *c,
     species=NULL;
   }
 
-
   /* Normalise etol (will be zero if not read) */
 
   e->etol/=m->n;
@@ -761,7 +766,27 @@ void qe_xml_read(FILE* infile, char *filename, struct unit_cell *c,
 	ts->m[i].forces=0;
     }
   }
-  
+
+  if (!strcmp(calculation,"bands")){
+    if (debug) fprintf(stderr,"Band path calc read\n");
+    if (e->eval){
+      e->path_eval=e->eval;
+      e->eval=NULL;
+    }
+    if (e->occ){
+      e->path_occ=e->occ;
+      e->occ=NULL;
+    }
+    if (k->n){
+      k->path_kpts=k->kpts;
+      k->kpts=NULL;
+      k->path_nkpt=k->n;
+      k->n=0;
+    }
+    e->path_nbands=e->nbands;
+    e->nbands=0;
+  }
+
   if ((flags&CHDEN)||(flags&SPINDEN)){
     i=strlen(filename);
     cptr2=filename+i;
