@@ -138,6 +138,7 @@ int super(struct unit_cell *c, struct contents *mtf,
       }
     rcell.vol=1.0/c->vol;
 
+    init_motif(&rmtf);
     rmtf.n=kp->n;
     rmtf.atoms=kp->kpts;
     addabs(rmtf.atoms,rmtf.n,rcell.basis);
@@ -159,7 +160,6 @@ int super(struct unit_cell *c, struct contents *mtf,
       }
 
       kp->n=rmtf.n;
-      free(kp->kpts);
       kp->kpts=rmtf.atoms;
       for(i=0;i<rmtf.n;i++) kp->kpts[i].wt*=fabs(new_cell.vol/c->vol);
     }
