@@ -155,7 +155,7 @@ void tube(struct unit_cell *c, struct contents *m, int rpt[3], double spacing){
       print_basis(new_basis);
     }
     
-    super(c,m,new_basis,NULL,NULL,NULL,0);
+    super(c,m,new_basis,NULL,NULL,NULL,0,NULL);
   }
   
   basis2abc(c->basis,abc);
