@@ -1,7 +1,7 @@
 /* Read a VASP CHGCAR or CHG, or also a POSCAR or a CONTCAR file
  */
 
-/* Copyright (c) 2017-2023 MJ Rutter 
+/* Copyright (c) 2017-2024 MJ Rutter 
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,7 +20,7 @@
 /* Format assumed to be:
  *
  * title (may be blank)
- * scale factor for axes (see below)
+ * scale factor for axes and Cartesian atomic coords (see below)
  * ax ay az
  * bx by bz
  * cx cy cz
@@ -72,7 +72,7 @@ void vasp_eigenval_read(FILE *infile, struct unit_cell *c, struct contents *m,
 void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
                struct contents *m, struct kpts *kp, struct grid *gptr,
                struct es *elect){
-  int i,j,k,nspec,natoms,frac,fft[3],ffts[3],isize,locpot,ns;
+  int i,j,k,ii,nspec,natoms,frac,fft[3],ffts[3],isize,locpot,elf,ns;
   int *nionsp,*species;
   char buffer[LINE_SIZE+1], *ptr,*p2;
   char *names[]={"POSCAR","CHGCAR","CONTCAR","CHG",NULL};
@@ -83,8 +83,9 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
   
   if (debug>2) fprintf(stderr,"vasp_read called\n");
 
-  locpot=0;
+  elf=locpot=0;
   if (strstr(filename,"LOCPOT")) locpot=1;
+  if (strstr(filename,"ELFCAR")) elf=1;
   
   /* title */
   
@@ -94,8 +95,11 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
   }
   /* Was the title more than spaces and an end-of-line? */
   ptr=buffer;
-  while((*ptr==' ')||(*ptr=='\n')) ptr++;
-  if (!strcasecmp(ptr,"comment\n")) *ptr=0;
+  while(isspace(*ptr)) ptr++;
+  p2=ptr+strlen(ptr)-1;
+  while ((p2>ptr)&&(isspace(*p2))) p2--;
+  *(p2+1)=0;
+  if (!strcasecmp(ptr,"comment")) *ptr=0;
   if (*ptr){
     m->title=malloc(strlen(buffer)+1);
     if (!m->title){
@@ -103,8 +107,6 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
       exit(1);
     }
     strcpy(m->title,buffer);
-    /* Remove newline */
-    m->title[strlen(m->title)-1]=0;
   }
 
   /* Scale factor */
@@ -150,6 +152,7 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
       for(j=0;j<3;j++)
         c->basis[i][j]*=vscale;
     real2rec(c);
+    ascale[2]=ascale[1]=ascale[0]=vscale;
   }
   
   /* Species, symbols */
@@ -238,6 +241,8 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
           fprintf(stderr,"Error parsing atom %d\n",k);
           exit(1);
         }
+	for(ii=0;ii<3;ii++)
+	  m->atoms[k].abs[ii]*=ascale[ii];
       }
       m->atoms[k].atno=species[i];
       k++;
@@ -308,7 +313,7 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
     return;
   }
 
-  if ((locpot==0)&&(!(flags&(CHDEN|SPINDEN)))) return;
+  if ((locpot==0)&&(elf==0)&&(!(flags&(CHDEN|SPINDEN)))) return;
 
   isize=fft[0]*fft[1]*fft[2];
 
@@ -321,7 +326,7 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
 
   /* If we wanted the charge density, now rescue onto a grid */
 
-  if ((locpot)||(flags&CHDEN)){
+  if ((locpot)||(elf)||(flags&CHDEN)){
     gptr=grid_new(gptr);
     
     if(!(gptr->data=malloc(isize*sizeof(double))))
@@ -342,24 +347,26 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
     scale=1/fabs(c->vol);
     if (flags&AU) scale*=BOHR*BOHR*BOHR;
 
-    if (!locpot){
-      if(!(flags&RAW)){
-	for(i=0;i<isize;i++) gptr->data[i]*=scale;
-	gptr->name="Density";
-	if (flags&AU)
-	  add_cmt(m->comment,"Densities in e Bohr^-3");
-	else
-	  add_cmt(m->comment,"Densities in e A^-3");
-      }
-      else{
-	gptr->name="Density_raw";
-	add_cmt(m->comment,"Densities are raw");
-      }
-    }
-    else{
+    if (locpot){
       gptr->name="Potential_eV";
     }
+    else if (elf){
+      gptr->name="ELF";
+    }
+    else if(!(flags&RAW)){
+      for(i=0;i<isize;i++) gptr->data[i]*=scale;
+      gptr->name="Density";
+      if (flags&AU)
+	add_cmt(m->comment,"Densities in e Bohr^-3");
+      else
+	add_cmt(m->comment,"Densities in e A^-3");
+    }
+    else{
+      gptr->name="Density_raw";
+      add_cmt(m->comment,"Densities are raw");
+    }
   }
+  
   free(dptr);
   dptr=NULL;
 
@@ -391,7 +398,7 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
 
   /* The only thing we might have, and know how to read, is a spin density */
   
-  if (!(flags&SPINDEN)) return;
+  if ((!(flags&SPINDEN))&&(!elf)) return;
   
   for(ns=0;ns<3;ns++){
 
@@ -414,7 +421,7 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
       dptr=realloc(dptr,j*sizeof(double));
       formatted_read(dptr,j,infile,NULL);
       if (!fgets(buffer,LINE_SIZE,infile)){
-	fprintf(stderr,"Spin density not found\n");
+	if (!elf) fprintf(stderr,"Spin density not found\n");
 	return;
       }
     }
@@ -453,7 +460,10 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
 	  (ffts[1]==fft[1])&&
 	  (ffts[2]==fft[2])){
 	if (debug) fprintf(stderr,"Spin density component found\n");
-	if (ns==0) gptr=grid_new(gptr);
+	if (ns==0) {
+	  if (elf) gptr->name="ELF_up";
+	  gptr=grid_new(gptr);
+	}
 
 	isize=fft[0]*fft[1]*fft[2];
 	if(!(gptr->data=realloc(gptr->data,isize*(ns+1)*sizeof(double))))
@@ -497,23 +507,28 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
   }
 
   /* Rescale spin density */
-  
-  scale=1/fabs(c->vol);
-  if (flags&AU) scale*=BOHR*BOHR*BOHR;
-        
-  if(!(flags&RAW)){
-    for(i=0;i<isize*gptr->comps;i++) gptr->data[i]*=scale;
-    if (gptr->comps==1)
-      gptr->name="Spin";
-    else
-      gptr->name="Vector spin";
-    if (flags&AU)
-      add_cmt(m->comment,"Spin in e Bohr^-3");
-    else
-      add_cmt(m->comment,"Spin in e A^-3");
+
+  if (elf){
+    gptr->name="ELF_down";
   }
   else{
-    gptr->name="Spin_raw";
+    scale=1/fabs(c->vol);
+    if (flags&AU) scale*=BOHR*BOHR*BOHR;
+        
+    if(!(flags&RAW)){
+      for(i=0;i<isize*gptr->comps;i++) gptr->data[i]*=scale;
+      if (gptr->comps==1)
+	gptr->name="Spin";
+      else
+	gptr->name="Vector spin";
+      if (flags&AU)
+	add_cmt(m->comment,"Spin in e Bohr^-3");
+      else
+	add_cmt(m->comment,"Spin in e A^-3");
+    }
+    else{
+      gptr->name="Spin_raw";
+    }
   }
   elect->nspins=2;
   if (gptr->comps==3) elect->nspinors=2;
@@ -521,7 +536,7 @@ void vasp_read(FILE* infile, char *filename, struct unit_cell *c,
 
 static int formatted_read(double *dptr,unsigned int len, FILE *infile,
                           char *first){
-  char *ptr,buffer[LINE_SIZE];
+  char *ptr,buffer[LINE_SIZE+1];
   int i,j;
   double dummy;
   
@@ -1038,7 +1053,8 @@ void vasp_eigenval_read(FILE *infile, struct unit_cell *c, struct contents *m,
   
   if (!c->basis){
     if (dict_get(m->dict,"in_dir")){
-      filename=malloc(strlen(dict_get(m->dict,"in_dir"))+7);
+      /* +8 for CONTCAR later */
+      filename=malloc(strlen(dict_get(m->dict,"in_dir"))+8);
       if (!filename) error_exit("malloc error for filename");
       strcpy(filename,dict_get(m->dict,"in_dir"));
       strcat(filename,"POSCAR");
@@ -1047,13 +1063,26 @@ void vasp_eigenval_read(FILE *infile, struct unit_cell *c, struct contents *m,
       filename="POSCAR";
 
     f=fopen(filename,"r");
+    if (!f){
+      if (debug) fprintf(stderr,"Failed to open %s\n",filename);
+      if (dict_get(m->dict,"in_dir")){
+	strcpy(filename,dict_get(m->dict,"in_dir"));
+	strcat(filename,"CONTCAR");
+      }
+      else{
+	filename="CONTCAR";
+      }
+      f=fopen(filename,"r");
+    }
+      
     if (f){
       fprintf(stderr,"Additionally reading %s\n",filename);
       vasp_read(f,filename,c,m,k,NULL,e);
       fclose(f);
     }
-    else
-      if (debug) fprintf(stderr,"Failed to open %s\n",filename);
+    else if (debug)
+      fprintf(stderr,"Failed to open %s\n",filename);
+    
     if (dict_get(m->dict,"in_dir")) free(filename);
   }
 
