@@ -128,6 +128,61 @@ void pv(struct vertex *v, int n){
   printf("\n");
 }
 
+void pplane(struct poly *p, int nn){
+  double vtmp[3],vtmp2[3],x,*len,*angle;
+  int i,j,hit,k,n;
+  struct plane *pl;
+  struct vertex *v;
+  struct line *l;
+
+  pl=p->p+nn;
+  v=p->v;
+  l=p->l;
+
+  n=0;
+  for(i=0;i<pl->n;i++)
+    if (l[pl->l[i]].valid) n++;
+  printf("Plane has %d sides\n",n);
+
+  len=malloc(pl->n*sizeof(double));
+  angle=malloc(pl->n*sizeof(double));
+  if ((!len)||(!angle)) error_exit("malloc error in pplane");
+  for(i=0;i<pl->n;i++){
+    if (!l[pl->l[i]].valid) continue;
+    for(j=0;j<3;j++)
+      vtmp[j]=v[l[pl->l[i]].v2].x[j]-v[l[pl->l[i]].v1].x[j];
+    len[i]=sqrt(vmod2(vtmp));
+    hit=0;
+    k=-1;
+    for(j=0;j<pl->n;j++){
+    if (!l[pl->l[j]].valid) continue;
+      if (j==i) continue;
+      if ((l[pl->l[j]].v1==l[pl->l[i]].v1)||(l[pl->l[j]].v2==l[pl->l[i]].v1)){
+	k=pl->l[j];
+	hit=1;
+	break;
+      }
+    }
+    if (!hit){
+      printf("Impossible\n");
+      free(angle);
+      free(len);
+      return;
+    }
+    for(j=0;j<3;j++)
+      vtmp2[j]=v[l[k].v2].x[j]-v[l[k].v1].x[j];
+    x=0;
+    for(j=0;j<3;j++)
+      x+=vtmp[j]*vtmp2[j];
+    x=x/sqrt(vmod2(vtmp)*vmod2(vtmp2));
+    angle[i]=acos(x);
+  }
+  for(i=0;i<pl->n;i++)
+    printf("len %f  angle %f\n",len[i],180*angle[i]/M_PI);
+  free(angle);
+  free(len);
+}
+
 void ppoly(struct poly *p){
   int i,j;
 
@@ -137,6 +192,7 @@ void ppoly(struct poly *p){
 	   i,p->p[i].norm[0],
 	   p->p[i].norm[1],p->p[i].norm[2],p->p[i].valid?"":"in");
     for(j=0;j<p->p[i].n;j++) printf("%d ",p->p[i].l[j]);
+    if (debug>1) {printf("\n"); pplane(p,i);}
   }
 
   printf("\n\n%d lines",p->nlines);
@@ -883,6 +939,8 @@ void zone1(double basis_in[3][3], struct poly *p){
     fprintf(stderr,
 	    "Initial volume %f, final %f, in zone1 (should be identical)\n",
 	    vol_init,vol_final);
+
+  if (debug>1) ppoly(p);
   
 }
 
@@ -1210,7 +1268,7 @@ void ibz(struct unit_cell *c, struct symmetry *s_in){
   if (!aeq(x/s->n,y))
     fprintf(stderr,"\n*** PROBLEM *** IBZ has unexpected volume!\n\n");
 
-  sym_free(s);
+  free_sym(s,1);
   
 }
 
